@@ -1,25 +1,31 @@
 #!/usr/bin/perl -T
-#
-# openwebmail-folder.pl - mail folder management program
-#
+#############################################################################
+# Open WebMail - Provides a web interface to user mailboxes                 #
+#                                                                           #
+# Copyright (C) 2001-2002                                                   #
+# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric                 #
+# Copyright (C) 2000                                                        #
+# Ernie Miller  (original GPL project: Neomail)                             #
+#                                                                           #
+# This program is distributed under GNU General Public License              #
+#############################################################################
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
-$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
-umask(0002); # make sure the openwebmail group can write
+umask(0007); # make sure the openwebmail group can write
 
 use strict;
 use Fcntl qw(:DEFAULT :flock);
-use CGI qw(-private_tempfiles :standard);
+use CGI qw(:standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "ow-shared.pl";
+require "openwebmail-shared.pl";
 require "filelock.pl";
 require "mime.pl";
 require "maildb.pl";
@@ -27,7 +33,7 @@ require "maildb.pl";
 use vars qw(%config %config_raw);
 use vars qw($thissession);
 use vars qw($loginname $domain $user $userrealname $uuid $ugid $homedir);
-use vars qw(%prefs %style %icontext);
+use vars qw(%prefs %style);
 use vars qw($folderdir @validfolders $folderusage);
 use vars qw($folder $printfolder $escapedfolder);
 
@@ -38,19 +44,16 @@ use vars qw($firstmessage);
 use vars qw($sort);
 
 $firstmessage = param("firstmessage") || 1;
-$sort = param("sort") || $prefs{'sort'} || 'date';
+$sort = param("sort") || $prefs{"sort"} || 'date';
 
 # extern vars
-use vars qw(%lang_folders %lang_text %lang_err);	# defined in lang/xy
-use vars qw($_OFFSET $_STATUS);				# defined in maildb.pl
+use vars qw($lang_charset %lang_folders %lang_text %lang_err);	# defined in lang/xy
 
 ########################## MAIN ##############################
 
 my $action = param("action");
 if ($action eq "editfolders") {
    editfolders();
-} elsif ($action eq "markreadfolder") {
-   markreadfolder();
 } elsif ($action eq "chkindexfolder") {
    reindexfolder(0);
 } elsif ($action eq "reindexfolder") {
@@ -78,10 +81,10 @@ sub editfolders {
    $total_allmessages=0;
    $total_foldersize=0;
 
-   push(@defaultfolders, 'INBOX',
-                         'saved-messages',
-                         'sent-mail',
-                         'saved-drafts',
+   push(@defaultfolders, 'INBOX', 
+                         'saved-messages', 
+                         'sent-mail', 
+                         'saved-drafts', 
                          'mail-trash');
 
    foreach (@validfolders) {
@@ -97,14 +100,21 @@ sub editfolders {
    my $html = '';
    my $temphtml;
 
-   $html=readtemplate("editfolders.template");
+   open (EDITFOLDERSTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/editfolders.template") or
+      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/editfolders.template!");
+   while (<EDITFOLDERSTEMPLATE>) {
+      $html .= $_;
+   }
+   close (EDITFOLDERSTEMPLATE);
+
    $html = applystyle($html);
 
    $html =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/g;
 
    printheader();
 
-   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $printfolder", qq|href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder"|). qq|\n|;
+   $temphtml = qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder" title="$lang_text{'backto'} $printfolder"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/backtofolder.gif" border="0" ALT="$lang_text{'backto'} $printfolder"></a>|;
+
    $html =~ s/\@\@\@MENUBARLINKS\@\@\@/$temphtml/g;
 
    $temphtml = start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl") .
@@ -128,7 +138,7 @@ sub editfolders {
 
    $temphtml = textfield(-name=>'foldername',
                          -default=>'',
-                         -size=> 24,
+                         -size=> $config{'foldername_maxlen'},
                          -maxlength=>$config{'foldername_maxlen'},
                          -override=>'1');
 
@@ -180,13 +190,16 @@ sub editfolders {
       $usagestr="&nbsp;";
    }
 
-   $total_foldersize=lenstr($total_foldersize,0);
-
+   if ($total_foldersize > 1048575){
+      $total_foldersize = int(($total_foldersize/1048576)+0.5) . "MB";
+   } elsif ($total_foldersize > 1023) {
+      $total_foldersize =  int(($total_foldersize/1024)+0.5) . "KB";
+   }
    $temphtml = qq|<tr>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$lang_text{'total'}</B></td>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$total_newmessages</B></td>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_allmessages</B></td>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_foldersize</B></td>|.
+               qq|<td align="center" bgcolor=$bgcolor><B>$total_allmessages</B></td>|.
+               qq|<td align="center" bgcolor=$bgcolor><B>$total_foldersize</B></td>|.
                qq|<td bgcolor=$bgcolor align="center">$usagestr</td>|.
                qq|</tr>|;
    $html =~ s/\@\@\@TOTAL\@\@\@/$temphtml/;
@@ -204,8 +217,8 @@ sub _folderline {
    my (%HDB, $newmessages, $allmessages, $foldersize);
    my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
 
-   if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
+   if ( -f "$headerdb$config{'dbm_ext'}" ) {
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
       dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       if ( defined($HDB{'ALLMESSAGES'}) ) {
          $allmessages=$HDB{'ALLMESSAGES'};
@@ -220,7 +233,7 @@ sub _folderline {
          $newmessages='&nbsp;';
       }
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
    } else {
       $allmessages='&nbsp;';
       $newmessages='&nbsp;';
@@ -230,7 +243,12 @@ sub _folderline {
    $foldersize = (-s "$folderfile") + (-s "$headerdb$config{'dbm_ext'}");
 
    $total_foldersize+=$foldersize;
-   $foldersize=lenstr($foldersize,0);
+   # round foldersize and change to an appropriate unit for display
+   if ($foldersize > 1048575){
+      $foldersize = int(($foldersize/1048576)+0.5) . "MB";
+   } elsif ($foldersize > 1023) {
+      $foldersize =  int(($foldersize/1024)+0.5) . "KB";
+   }
 
    my $escapedcurrfolder = escapeURL($currfolder);
    my $url = "$config{'ow_cgiurl'}/openwebmail-folder.pl?sessionid=$thissession&amp;folder=$escapedcurrfolder&amp;action=downloadfolder";
@@ -241,11 +259,11 @@ sub _folderline {
                 qq|<td align="center" bgcolor=$bgcolor>|.
                 qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedcurrfolder">|.
                 qq|$folderstr</a>&nbsp;\n|.
-                iconlink("download.gif", "$lang_text{'download'} $folderstr", qq|href="$url"|).
-                qq|</td>\n|.
+                qq|<a href="$url" title="$lang_text{'download'} $folderstr"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/download.gif" align="absmiddle" border="0" ALT="$lang_text{'download'} $folderstr">|.
+                qq|</a></td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
-                qq|<td align="center" bgcolor=$bgcolor>&nbsp;$allmessages</td>|.
-                qq|<td align="center" bgcolor=$bgcolor>&nbsp;$foldersize</td>\n|;
+                qq|<td align="center" bgcolor=$bgcolor>$allmessages</td>|.
+                qq|<td align="center" bgcolor=$bgcolor>$foldersize</td>\n|;
 
    $temphtml .= qq|<td bgcolor=$bgcolor align="center">\n|;
 
@@ -274,24 +292,19 @@ sub _folderline {
                        -override=>'1');
    $temphtml .= "\n";
 
-   my $jsfolderstr=$lang_folders{$currfolder}||$currfolder;
-   $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
-   $temphtml .= submit(-name=>"$lang_text{'markread'}",
-                       -class=>"medtext",
-                       -onClick=>"return OpConfirm('folderform$i', 'markreadfolder', $lang_text{'foldermarkreadconf'}+' ( $jsfolderstr )')");
    $temphtml .= submit(-name=>"$lang_text{'chkindex'}",
                        -class=>"medtext",
-                       -onClick=>"return OpConfirm('folderform$i', 'chkindexfolder', $lang_text{'folderchkindexconf'}+' ( $jsfolderstr )')");
-   $temphtml .= submit(-name=>"$lang_text{'reindex'}",
+                       -onClick=>"return OpConfirm('folderform$i', 'chkindexfolder', $lang_text{'folderchkindexconf'}+' ( $currfolder )')");
+   $temphtml .= submit(-name=>"$lang_text{'reindex'}", 
                        -class=>"medtext",
-                       -onClick=>"return OpConfirm('folderform$i', 'reindexfolder', $lang_text{'folderreindexconf'}+' ( $jsfolderstr )')");
+                       -onClick=>"return OpConfirm('folderform$i', 'reindexfolder', $lang_text{'folderreindexconf'}+' ( $currfolder )')");
    if ($currfolder ne "INBOX") {
-      $temphtml .= submit(-name=>"$lang_text{'rename'}",
+      $temphtml .= submit(-name=>"$lang_text{'rename'}", 
                           -class=>"medtext",
-                          -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $jsfolderstr )')");
+                          -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $currfolder )')");
       $temphtml .= submit(-name=>"$lang_text{'delete'}",
                           -class=>"medtext",
-                          -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $jsfolderstr )')");
+                          -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $currfolder )')");
    }
 
    $temphtml .= "</td></tr>";
@@ -301,93 +314,12 @@ sub _folderline {
 }
 ################### END EDITFOLDERS ########################
 
-################### MARKREADFOLDER ##############################
-sub markreadfolder {
-   my $foldertomark = param('foldername') || '';
-   $foldertomark =~ s!\.\.+/!!g;
-   $foldertomark =~ s!^\s*/!!g;
-   $foldertomark =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
-   ($foldertomark =~ /^(.+)$/) && ($foldertomark = $1);
-   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertomark);
-
-   filelock($folderfile, LOCK_EX|LOCK_NB) or openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
-
-   if (update_headerdb($headerdb, $folderfile)<0) {
-      filelock($folderfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
-   }
-
-   my (%HDB, %offset, %status);
-   filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
-   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
-   foreach my $messageid (keys %HDB) {
-      next if ( $messageid eq 'METAINFO'
-             || $messageid eq 'NEWMESSAGES'
-             || $messageid eq 'INTERNALMESSAGES'
-             || $messageid eq 'ALLMESSAGES'
-             || $messageid eq "" );
-      my @attr=split( /@@@/, $HDB{$messageid} );
-      if ($attr[$_STATUS] !~ /R/i) {
-         $offset{$messageid}=$attr[$_OFFSET];
-         $status{$messageid}=$attr[$_STATUS];
-      }
-   }
-   dbmclose(%HDB);
-   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
-
-   my @markids;
-   my $tmpfile="/tmp/markread_tmp_$$";
-   my $tmpdb="/tmp/.markread_tmp_$$";
-   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);
-   ($tmpdb =~ /^(.+)$/) && ($tmpdb = $1);
-
-   filelock("$tmpfile", LOCK_EX);
-
-   if (update_headerdb($tmpdb, $tmpfile)<0) {
-      filelock($tmpfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
-   }
-
-   foreach my $messageid (sort { $offset{$a}<=>$offset{$b} } keys %offset) {
-      my @copyid;
-      push(@copyid, $messageid);
-      if (operate_message_with_ids("copy", \@copyid,
-   					$folderfile, $headerdb, $tmpfile, $tmpdb) >0 ) {
-         update_message_status($messageid, $status{$messageid}."R", $tmpdb, $tmpfile);
-         push(@markids, $messageid);
-
-         if ($#markids>=99) { # flush per 100 msgs
-            operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
-            operate_message_with_ids("move", \@markids,
-   					$tmpfile, $tmpdb, $folderfile, $headerdb);
-            @markids=();
-         }
-
-      }
-   }
-   operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
-   operate_message_with_ids("move", \@markids,
-   					$tmpfile, $tmpdb, $folderfile, $headerdb);
-
-   filelock("$tmpfile", LOCK_UN);
-   filelock($folderfile, LOCK_UN);
-   unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
-
-   writelog("markread folder - $foldertomark");
-   writehistory("markread folder - $foldertomark");
-
-   getfolders(\@validfolders, \$folderusage);
-   editfolders();
-}
-################### END MARKREADFOLDER ##########################
-
 ################### REINDEXFOLDER ##############################
 sub reindexfolder {
    my $recreate=$_[0];
    my $foldertoindex = param('foldername') || '';
-   $foldertoindex =~ s!\.\.+/!!g;
-   $foldertoindex =~ s!^\s*/!!g;
-   $foldertoindex =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
+   $foldertoindex =~ s/\.\.+//g;
+   $foldertoindex =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
    ($foldertoindex =~ /^(.+)$/) && ($foldertoindex = $1);
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoindex);
 
@@ -399,37 +331,26 @@ sub reindexfolder {
 
    if ( -f "$headerdb$config{'dbm_ext'}" ) {
       my %HDB;
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
-      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       $HDB{'METAINFO'}={'RENEW'};
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
-   }
-   if (update_headerdb($headerdb, $folderfile)<0) {
-      filelock($folderfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
    }
+   update_headerdb($headerdb, $folderfile);
 
-   if ($recreate) {
-      writelog("reindex folder - $foldertoindex");
-      writehistory("reindex folder - $foldertoindex");
-   } else {
-      writelog("chkindex folder - $foldertoindex");
-      writehistory("chkindex folder - $foldertoindex");
-   }
-
-#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&firstmessage=$firstmessage\n\n";
    getfolders(\@validfolders, \$folderusage);
    editfolders();
+
+#   print "Location: $config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&firstmessage=$firstmessage\n\n";
 }
 ################### END REINDEXFOLDER ##########################
 
 ################### ADDFOLDER ##############################
 sub addfolder {
    my $foldertoadd = param('foldername') || '';
-   $foldertoadd =~ s!\.\.+/!!g;
-   $foldertoadd =~ s!^\s*/!!g;
-   $foldertoadd =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
+   $foldertoadd =~ s/\.\.+//g;
+   $foldertoadd =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
    ($foldertoadd =~ /^(.+)$/) && ($foldertoadd = $1);
 
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
@@ -464,7 +385,7 @@ sub addfolder {
 sub is_defaultfolder {
    my $foldername=$_[0];
    if ($foldername eq 'INBOX' ||
-       $foldername eq 'saved-messages' ||
+       $foldername eq 'saved-messages' || 
        $foldername eq 'sent-mail' ||
        $foldername eq 'saved-drafts' ||
        $foldername eq 'mail-trash' ||
@@ -483,9 +404,8 @@ sub is_defaultfolder {
 ################### DELETEFOLDER ##############################
 sub deletefolder {
    my $foldertodel = param('foldername') || '';
-   $foldertodel =~ s!\.\.+/!!g;
-   $foldertodel =~ s!^\s*/!!g;
-   $foldertodel =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
+   $foldertodel =~ s/\.\.+//g;
+   $foldertodel =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
    ($foldertodel =~ /^(.+)$/) && ($foldertodel = $1);
 
    # if is INBOX, return to editfolder immediately
@@ -495,16 +415,14 @@ sub deletefolder {
    }
 
    if ( -f "$folderdir/$foldertodel" ) {
-      my $headerdb="$folderdir/$foldertodel";
-      ($headerdb =~ /^(.+)\/(.*)$/) && ($headerdb = "$1/.$2");
       unlink ("$folderdir/$foldertodel",
+              "$folderdir/.$foldertodel$config{'dbm_ext'}",
+              "$folderdir/.$foldertodel.db",
+	      "$folderdir/.$foldertodel.dir",
+              "$folderdir/.$foldertodel.pag",
+              "$folderdir/.$foldertodel.cache",
               "$folderdir/$foldertodel.lock",
-              "$folderdir/$foldertodel.lock.lock",
-              "$headerdb$config{'dbm_ext'}",
-              "$headerdb.db",
-	      "$headerdb.dir",
-              "$headerdb.pag",
-              "$headerdb.cache");
+              "$folderdir/$foldertodel.lock.lock");              
 
       writelog("delete folder - $foldertodel");
       writehistory("delete folder - $foldertodel");
@@ -518,9 +436,8 @@ sub deletefolder {
 ################### RENAMEFOLDER ##############################
 sub renamefolder {
    my $oldname = param('foldername') || '';
-   $oldname =~ s!\.\.+/!!g;
-   $oldname =~ s!^\s*/!!g;
-   $oldname =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
+   $oldname =~ s/\.\.+//g;
+   $oldname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
    ($oldname =~ /^(.+)$/) && ($oldname = $1);
 
    if ($oldname eq 'INBOX') {
@@ -529,9 +446,8 @@ sub renamefolder {
    }
 
    my $newname = param('foldernewname');
-   $newname =~ s!\.\.+/!!g;
-   $newname =~ s!^\s*/!!g;
-   $newname =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
+   $newname =~ s/\.\.+//g;
+   $newname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
    ($newname =~ /^(.+)$/) && ($newname = $1);
 
    if (length($newname) > $config{'foldername_maxlen'}) {
@@ -546,17 +462,13 @@ sub renamefolder {
    }
 
    if ( -f "$folderdir/$oldname" ) {
-      my $oldheaderdb="$folderdir/$oldname";
-      my $newheaderdb="$folderdir/$newname";
-      ($oldheaderdb =~ /^(.+)\/(.*)$/) && ($oldheaderdb = "$1/.$2");
-      ($newheaderdb =~ /^(.+)\/(.*)$/) && ($newheaderdb = "$1/.$2");
-      rename("$folderdir/$oldname",            "$folderdir/$newname");
-      rename("$oldheaderdb$config{'dbm_ext'}", "$newheaderdb$config{'dbm_ext'}");
-      rename("$oldheaderdb.db",                "$newheaderdb.db");
-      rename("$oldheaderdb.dir",               "$newheaderdb.dir");
-      rename("$oldheaderdb.pag",               "$newheaderdb.pag");
-      rename("$oldheaderdb.cache",             "$newheaderdb.cache");
-      unlink("$folderdir/$newname.lock",       "$folderdir/$oldname.lock.lock");
+      rename("$folderdir/$oldname",          "$folderdir/$newname");
+      rename("$folderdir/.$oldname$config{'dbm_ext'}", "$folderdir/.$newname$config{'dbm_ext'}");
+      rename("$folderdir/.$oldname.db",      "$folderdir/.$newname.db");
+      rename("$folderdir/.$oldname.dir",     "$folderdir/.$newname.dir");
+      rename("$folderdir/.$oldname.pag",     "$folderdir/.$newname.pag");
+      rename("$folderdir/.$oldname.cache",   "$folderdir/.$newname.cache");
+      unlink("$folderdir/$oldname.lock", "$folderdir/$oldname.lock.lock");
 
       writelog("rename folder - rename $oldname to $newname");
       writehistory("rename folder - rename $oldname to $newname");
@@ -575,12 +487,12 @@ sub downloadfolder {
    my $buff;
 
    if ( -x '/usr/local/bin/zip' ) {
-      $cmd="/usr/local/bin/zip -rq - $folderfile |";
+      $cmd="/usr/local/bin/zip -r - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
    } elsif ( -x '/usr/bin/zip' ) {
-      $cmd="/usr/bin/zip -rq - $folderfile |";
+      $cmd="/usr/bin/zip -r - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
