@@ -1,49 +1,42 @@
 #!/usr/bin/perl -T
 #
-# openwebmail-spell.pl - spell check program
+# spell check program by tung@turtle.ee.ncku.edu.tw
+# modified from WBOSS Version 1.50a 
 #
-# 2001/09/27 tung@turtle.ee.ncku.edu.tw
-#            modified from WBOSS Version 1.50a
-#
-# WBOSS is available at http://www.dontpokebadgers.com/spellchecker/
+# WBOSS is available at http://www.dontpokebadgers.com/spellchecker/ 
 # and is copyrighted by 2001, Joshua Cantara
 #
 
 # This is the table of valid letters for various dictionaries.
-# If your dictionary checks vocabularies composed by characters other
+# If your dictionary checks vocabularies composed by characters other 
 # than english letters, you have to define new entry in below hash
 
 my %dictionary_letters =
    (
    english   => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
-   br        => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
-   czech     => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
-   dansk     => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
-   deutsch   => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
    magyar    => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz~',
-   polski    => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz󶼿ʣӦ',
    ukrainian => 'ȧƦ֤\'',
+   deutsch   => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
    );
 
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
-$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
-umask(0002); # make sure the openwebmail group can write
+umask(0007); # make sure the openwebmail group can write
 
 use strict;
 use Fcntl qw(:DEFAULT :flock);
 use IPC::Open3;
-use CGI qw(-private_tempfiles :standard);
+use CGI qw(:standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "ow-shared.pl";
+require "openwebmail-shared.pl";
 require "filelock.pl";
 
 use vars qw(%config %config_raw);
@@ -95,7 +88,7 @@ if (defined(param('string'))) {
    final($form,$field);
 } else {
    printheader();
-   print "What the heck? Invalid input for Spellcheck!";
+   print "What the heck? Inavlid input for Spellcheck!";
    printfooter(1);
 }
 
@@ -115,7 +108,13 @@ sub docheck {
    my $escapedwordframe;
    local $_;
 
-   $html=readtemplate("spellcheck.template");
+   open (SPELLCHECKTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/spellcheck.template") or
+      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/spellcheck.template");
+   while (<SPELLCHECKTEMPLATE>) {
+      $html .= $_;
+   }
+   close (IMPORTTEMPLATE);
+
    $html = applystyle($html);
 
    $html =~ s/\@\@\@FORMNAME\@\@\@/$formname/;
@@ -127,7 +126,7 @@ sub docheck {
 
    # escapedwordframe must be done after words2html()
    # since $wordframe may changed in words2html()
-   $escapedwordframe=escapeURL($wordframe);
+   $escapedwordframe=escapeURL($wordframe);	
 
    $temphtml = startform(-action=>"$config{'ow_cgiurl'}/openwebmail-spell.pl",
                          -name=>'spellcheck') .
@@ -191,7 +190,7 @@ sub final {
 
    $escapedfinalstring=words2text();
 
-   # since jscript has problem in unescape doublebyte char string,
+   # since jscript has problem in unescape doublebyte char string, 
    # we only escape " to !QUOT! and unescape in jscript by RegExp
    # $escapedfinalstring=escapeURL(words2text());
    $escapedfinalstring=~s/"/!QUOT!/g;
@@ -206,7 +205,7 @@ sub final {
    <!--
    updateclose();
 
-   function updateclose()
+   function updateclose() 
    {
       var quot = new RegExp("!QUOT!","g");
 
@@ -243,16 +242,10 @@ sub text2words {
    local $_;
 
    # init don't care term
-   $wordignore  = "they'll we'll you'll she'll he'll i'll ".
-                  "they've we've you've I've ".
-                  "can't couldn't won't wouldn't shouldn't ".
-                  "don't doesn't didn't hasn't hadn't ".
-                  "isn't wasn't aren't weren't ".
-                  "http https ftp nntp smtp nfs html xml sgml mailto ".
-                  "freebsd linux solaris gnu gpl bsd openwebmail";
+   $wordignore="http https ftp nntp smtp nfs html xml sgml mailto freebsd linux solaris gnu gpl bsd openwebmail";
 
    # put url to ignore
-   foreach ($text=~m![A-Za-z]+tp://[A-Za-z\d\.]+!ig) {
+   foreach ($text=~m![A-Za-z]+tp://[A-Za-z\d\.]+!ig) {	
       $wordignore.=" $_";
    }
    # put email to ignore
@@ -269,7 +262,7 @@ sub text2words {
    $wordframe=$text;
    $wordframe=~s/([$dicletters][$dicletters\-]*[$dicletters])|(~~[$dicletters][$dicletters\-]*[$dicletters])/_word2label($1)/ge;
    return $wordcount;
-}
+}   
 
 # fill $wordframe and @words by CGI $query
 sub cgiparam2words {
@@ -312,7 +305,7 @@ sub words2html {
 
       if ( $words[$i]=~/^~~/ ) {	# check if manualfix
          my $origword=substr($words[$i],2);
-         my $len=length($origword);
+         my $len=length($origword);    
          $wordhtml=qq|<input type="text" size="$len" name="$i" value="$origword">\n|;
          $worderror++;
 
@@ -325,7 +318,7 @@ sub words2html {
             $worderror++;
 
          } elsif ($r->{'type'} eq 'miss')  {
-            my $sugg;
+            my $sugg; 
             $wordhtml=qq|<select size="1" name="$i">\n|.
                       qq|<option>$words[$i]</option>\n|.
                       qq|<option value="~~$words[$i]">--$lang_text{'manuallyfix'}--</option>\n|;
