@@ -26,11 +26,17 @@
 #
 # 2001/12/21 tung@turtle.ee.ncku.edu.tw
 #
-
+use strict;
 use Fcntl qw(:DEFAULT :flock);
 use FileHandle;
 
-# CONSTANT, message attribute number
+use vars qw($_OFFSET $_FROM $_TO $_DATE $_SUBJECT $_CONTENT_TYPE $_STATUS $_SIZE $_REFERENCES);
+use vars qw(%month %timezones);
+
+# extern vars
+use vars qw(%config);	# defined in caller openwebmail-xxx.pl
+
+# message attribute number, CONST
 ($_OFFSET, $_FROM, $_TO, $_DATE, $_SUBJECT, $_CONTENT_TYPE, $_STATUS, $_SIZE, $_REFERENCES)
  =(0,1,2,3,4,5,6,7,8);
 
@@ -52,91 +58,91 @@ use FileHandle;
 # @monthday=qw(31 31 29 31 30 31 30 31 31 30 31 30 31);
 
 %timezones = qw(ACDT +1030
-                   ACST +0930
-                   ADT  -0300
-                   AEDT +1100
-                   AEST +1000
-                   AHDT -0900
-                   AHST -1000
-                   AST  -0400
-                   AT   -0200
-                   AWDT +0900
-                   AWST +0800
-                   AZST +0400
-                   BAT  +0300
-                   BDST +0200
-                   BET  -1100
-                   BST  -0300
-                   BT   +0300
-                   BZT2 -0300
-                   CADT +1030
-                   CAST +0930
-                   CAT  -1000
-                   CCT  +0800
-                   CDT  -0500
-                   CED  +0200
-                   CET  +0100
-                   CST  -0600
-                   EAST +1000
-                   EDT  -0400
-                   EED  +0300
-                   EET  +0200
-                   EEST +0300
-                   EST  -0500
-                   FST  +0200
-                   FWT  +0100
-                   GMT  +0000
-                   GST  +1000
-                   HDT  -0900
-                   HST  -1000
-                   IDLE +1200
-                   IDLW -1200
-                   IST  +0530
-                   IT   +0330
-                   JST  +0900
-                   JT   +0700
-                   MDT  -0600
-                   MED  +0200
-                   MET  +0100
-                   MEST +0200
-                   MEWT +0100
-                   MST  -0700
-                   MT   +0800
-                   NDT  -0230
-                   NFT  -0330
-                   NT   -1100
-                   NST  +0630
-                   NZ   +1100
-                   NZST +1200
-                   NZDT +1300
-                   NZT  +1200
-                   PDT  -0700
-                   PST  -0800
-                   ROK  +0900
-                   SAD  +1000
-                   SAST +0900
-                   SAT  +0900
-                   SDT  +1000
-                   SST  +0200
-                   SWT  +0100
-                   USZ3 +0400
-                   USZ4 +0500
-                   USZ5 +0600
-                   USZ6 +0700
-                   UT   +0000
-                   UTC  +0000
-                   UZ10 +1100
-                   WAT  -0100
-                   WET  +0000
-                   WST  +0800
-                   YDT  -0800
-                   YST  -0900
-                   ZP4  +0400
-                   ZP5  +0500
-                   ZP6  +0600);
+                ACST +0930
+                ADT  -0300
+                AEDT +1100
+                AEST +1000
+                AHDT -0900
+                AHST -1000
+                AST  -0400
+                AT   -0200
+                AWDT +0900
+                AWST +0800
+                AZST +0400
+                BAT  +0300
+                BDST +0200
+                BET  -1100
+                BST  -0300
+                BT   +0300
+                BZT2 -0300
+                CADT +1030
+                CAST +0930
+                CAT  -1000
+                CCT  +0800
+                CDT  -0500
+                CED  +0200
+                CET  +0100
+                CST  -0600
+                EAST +1000
+                EDT  -0400
+                EED  +0300
+                EET  +0200
+                EEST +0300
+                EST  -0500
+                FST  +0200
+                FWT  +0100
+                GMT  +0000
+                GST  +1000
+                HDT  -0900
+                HST  -1000
+                IDLE +1200
+                IDLW -1200
+                IST  +0530
+                IT   +0330
+                JST  +0900
+                JT   +0700
+                MDT  -0600
+                MED  +0200
+                MET  +0100
+                MEST +0200
+                MEWT +0100
+                MST  -0700
+                MT   +0800
+                NDT  -0230
+                NFT  -0330
+                NT   -1100
+                NST  +0630
+                NZ   +1100
+                NZST +1200
+                NZDT +1300
+                NZT  +1200
+                PDT  -0700
+                PST  -0800
+                ROK  +0900
+                SAD  +1000
+                SAST +0900
+                SAT  +0900
+                SDT  +1000
+                SST  +0200
+                SWT  +0100
+                USZ3 +0400
+                USZ4 +0500
+                USZ5 +0600
+                USZ6 +0700
+                UT   +0000
+                UTC  +0000
+                UZ10 +1100
+                WAT  -0100
+                WET  +0000
+                WST  +0800
+                YDT  -0800
+                YST  -0900
+                ZP4  +0400
+                ZP5  +0500
+                ZP6  +0600);
 
 if ( $config{'dbm_ext'} eq "" ) {
-   $config{'dbm_ext'}="db";
+   $config{'dbm_ext'}=".db";
 }
 
 ######################### UPDATE_HEADERDB ############################
@@ -147,12 +153,12 @@ sub update_headerdb {
    my (%HDB, %OLDHDB);
    my @oldmessageids=();
 
-   ($headerdb =~ /^(.+)$/) && ($headerdb = $1);		# bypass taint check
+   ($headerdb =~ /^(.+)$/) && ($headerdb = $1);		# untaint ...
    if ( -e "$headerdb$config{'dbm_ext'}" ) {
       my ($metainfo, $allmessages, $internalmessages, $newmessages);
 
       filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%HDB, $headerdb, undef);
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       $metainfo=$HDB{'METAINFO'};
       $allmessages=$HDB{'ALLMESSAGES'};
       $internalmessages=$HDB{'INTERNALMESSAGES'};
@@ -175,7 +181,7 @@ sub update_headerdb {
       # we will try to reference records in old headerdb if possible
       @oldmessageids=get_messageids_sorted_by_offset("$headerdb.old");
       filelock("$headerdb.old$config{'dbm_ext'}", LOCK_SH);
-      dbmopen(%OLDHDB, "$headerdb.old", undef);
+      dbmopen(%OLDHDB, "$headerdb.old$config{'dbmopen_ext'}", undef);
    }
 
    my $messagenumber = -1;
@@ -188,12 +194,12 @@ sub update_headerdb {
 
    my @duplicateids=();
 
-   my ($line, $lastline, $lastheader, $namedatt_count, $zhtype);
+   my ($line, $lastheader, $namedatt_count, $zhtype);
    my ($_message_id, $_offset);
    my ($_from, $_to, $_date, $_subject);
    my ($_content_type, $_status, $_messagesize, $_references, $_inreplyto);
 
-   dbmopen(%HDB, $headerdb, 0600);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
    filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
    %HDB=();	# ensure the headerdb is empty
 
@@ -227,7 +233,6 @@ sub update_headerdb {
    }
    seek(FOLDER, $totalsize, 0);		# set file ptr back to totalsize
 
-   $lastline="\r";
    while (defined($line = <FOLDER>)) {
 
       $offset=$totalsize;
@@ -236,8 +241,7 @@ sub update_headerdb {
       # ex: From tung@turtle.ee.ncku.edu.tw Fri Jun 22 14:15:33 2001
       # ex: From tung@turtle.ee.ncku.edu.tw Mon Aug 20 18:24 CST 2001
       # ex: From nsb@thumper.bellcore.com Wed Mar 11 16:27:37 EST 1992
-      if ( $lastline =~ /^\r*$/ &&
-           $line =~ /^From .*(\w\w\w)\s+(\w\w\w)\s+(\d+)\s+(\d+):(\d+):?(\d*)\s+(\w\w\w\s+)?(\d\d+)/ ) {
+      if ( $line =~ /^From .*(\w\w\w)\s+(\w\w\w)\s+(\d+)\s+(\d+):(\d+):?(\d*)\s+(\w\w\w\s+)?(\d\d+)/ ) {
          if ($_messagesize >0) {
 
             $_from=~s/\@\@/\@\@ /g;         $_from=~s/\@$/\@ /;
@@ -256,12 +260,16 @@ sub update_headerdb {
 	    $_references =~ s/\s{2,}/ /g;
 
             if ($_message_id eq '') {	# fake messageid with date and from
-               $_message_id="$_date.".(email2nameaddr($_from))[1];
+               if ($_date ne "") {
+                  $_message_id="$_date.".(email2nameaddr($_from))[1];
+               } else {
+                  $_message_id = getdateserial().'.X'.int(rand()*100000);
+               }
                $_message_id=~s![\<\>\(\)\s\/"':]!!g;
                $_message_id="<$_message_id>";
             }
 
-            # flags flag used by openwebmail internally
+            # flags used by openwebmail internally
             $_status .= "T" if ($namedatt_count>0);
             $_status .= "B" if ($zhtype eq 'big5');
             $_status .= "G" if ($zhtype eq 'gb');
@@ -298,7 +306,7 @@ sub update_headerdb {
             if ($line =~ /^\r*$/) {
                $inheader = 0;
 
-               ### Convert to readable text from MIME-encoded
+               # Convert to readable text from MIME-encoded
                $_from = decode_mimewords($_from);
                $_subject = decode_mimewords($_subject);
 
@@ -338,7 +346,7 @@ sub update_headerdb {
                      $_messagesize=$oldmsgsize;
                      $totalsize=$_offset+$_messagesize;
                      # copy vars related to content
-                     $namedatt_account++ if ($oldstatus=~/T/);
+                     $namedatt_count++   if ($oldstatus=~/T/);
                      $zhtype="big5"      if ($oldstatus=~/B/);
                      $zhtype="gb"        if ($oldstatus=~/G/);
                   }
@@ -412,7 +420,6 @@ sub update_headerdb {
          }
       }
 
-      $lastline=$line;
    }
 
    # Catch the last message, since there won't be a From: to trigger the capture
@@ -488,7 +495,7 @@ sub get_messageids_sorted_by_offset {
    my (%HDB, @attr, %offset, $key, $data);
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
 
    while ( ($key, $data)=each(%HDB) ) {
       next if ( $key eq 'METAINFO' 
@@ -545,7 +552,7 @@ sub get_info_messageids_sorted {
    }
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    $metainfo=$HDB{'METAINFO'};
    dbmclose(%HDB);
    filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
@@ -565,7 +572,7 @@ sub get_info_messageids_sorted {
    if ( $cache_metainfo ne $metainfo || $cache_headerdb ne $headerdb ||
         $cache_sort ne $sort || $cache_ignore_internal ne $ignore_internal || 
         $totalsize=~/[^\d]/ ) { 
-      ($cachefile =~ /^(.+)$/) && ($cachefile = $1);		# bypass taint check
+      ($cachefile =~ /^(.+)$/) && ($cachefile = $1);		# untaint ...
       open(CACHE, ">$cachefile");
       print CACHE $metainfo, "\n", $headerdb, "\n", $sort, "\n", $ignore_internal, "\n";
       if ( $sort eq 'date' ) {
@@ -634,7 +641,7 @@ sub get_info_messageids_sorted_by_date {
    my @messageids;
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    while ( ($key, $data)=each(%HDB) ) {
       if ( $key eq 'METAINFO' ||
            $key eq 'ALLMESSAGES' ||
@@ -665,7 +672,7 @@ sub get_info_messageids_sorted_by_from {
    my @messageids;
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    while ( ($key, $data)=each(%HDB) ) {
       if ( $key eq 'METAINFO' ||
            $key eq 'ALLMESSAGES' ||
@@ -719,7 +726,7 @@ sub get_info_messageids_sorted_by_to {
    my @messageids;
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    while ( ($key, $data)=each(%HDB) ) {
       if ( $key eq 'METAINFO' ||
            $key eq 'ALLMESSAGES' ||
@@ -776,7 +783,7 @@ sub get_info_messageids_sorted_by_subject {
    my (@message_ids, @message_depths);
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    while ( ($key, $data)=each(%HDB) ) {
       if ( $key eq 'METAINFO' ||
            $key eq 'ALLMESSAGES' ||
@@ -852,23 +859,30 @@ sub get_info_messageids_sorted_by_subject {
    }
 
    # Finally, we recursively traverse the tree.
-   sub recursively_thread {
-      my ($id, $depth) = @_;
-      push @message_ids, $id;
-      push @message_depths, $depth;
-      my @children = sort { $dateserial{$a} <=> $dateserial{$b}; } @{$thread_children{$id}};
-      foreach my $thread (@children) {
-         recursively_thread ($thread, $depth+1);
-      }
-   }
-
    @thread_roots = sort { $dateserial{$a} <=> $dateserial{$b}; } @thread_roots;
    foreach my $key (@thread_roots) {
-      recursively_thread ($key, 0);
+      _recursively_thread ($key, 0, 
+		\@message_ids, \@message_depths, \%thread_children, \%dateserial);
    }
    return($totalsize, $new, \@message_ids, \@message_depths);
 }
 
+sub _recursively_thread {
+   my ($id, $depth,  
+	$r_message_ids, $r_message_depths, $r_thread_children, $r_dateserial) = @_;
+
+   push @{$r_message_ids}, $id;
+   push @{$r_message_depths}, $depth;
+   if (defined(${$r_thread_children}{$id})) {
+      my @children = sort { ${$r_dateserial}{$a} <=> ${$r_dateserial}{$b}; } @{${$r_thread_children}{$id}};
+      foreach my $thread (@children) {
+         _recursively_thread ($thread, $depth+1, 
+	 $r_message_ids, $r_message_depths, $r_thread_children, $r_dateserial);
+      }
+   }
+}
+
+
 sub get_info_messageids_sorted_by_size {
    my ($headerdb, $ignore_internal)=@_;
    my (%HDB, @attr, %size, %dateserial, $key, $data);
@@ -876,7 +890,7 @@ sub get_info_messageids_sorted_by_size {
    my @messageids;
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    while ( ($key, $data)=each(%HDB) ) {
       if ( $key eq 'METAINFO' ||
            $key eq 'ALLMESSAGES' ||
@@ -910,7 +924,7 @@ sub get_info_messageids_sorted_by_status {
    my @messageids;
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    while ( ($key, $data)=each(%HDB) ) {
       if ( $key eq 'METAINFO' ||
            $key eq 'ALLMESSAGES' ||
@@ -948,8 +962,6 @@ sub get_info_messageids_sorted_by_status {
                     } keys(%status);
    return($totalsize, $new, \@messageids);
 }
-
-
 ############### END GET_MESSAGEIDS_SORTED_BY_...  #################
 
 ####################### GET_MESSAGE_.... ###########################
@@ -958,14 +970,13 @@ sub get_message_attributes {
    my (%HDB, @attr);
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen(%HDB, $headerdb, undef);
+   dbmopen(%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    @attr=split(/@@@/, $HDB{$messageid} );
    dbmclose(%HDB);
    filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
    return(@attr);
 }
 
-
 sub get_message_block {
    my ($messageid, $headerdb, $folderhandle)=@_;
    my (@attr, $buff);
@@ -982,7 +993,6 @@ sub get_message_block {
    return(\$buff);
 }
 
-
 sub get_message_header {
    my ($messageid, $headerdb, $folderhandle)=@_;
    my (@attr, $header);
@@ -1005,7 +1015,6 @@ sub get_message_header {
 
 ###################### END GET_MESSAGE_.... ########################
 
-
 ###################### UPDATE_MESSAGE_STATUS ########################
 sub update_message_status {
    my ($messageid, $status, $headerdb, $folderfile) = @_;
@@ -1021,7 +1030,7 @@ sub update_message_status {
    my $i;
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
-   dbmopen (%HDB, $headerdb, 0600);
+   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
 
    for ($i=0; $i<=$#messageids; $i++) {
       if ($messageids[$i] eq $messageid) {
@@ -1158,11 +1167,11 @@ sub operate_message_with_ids {
    my $counted=0;
    
    filelock("$srcdb$config{'dbm_ext'}", LOCK_EX);
-   dbmopen (%HDB, $srcdb, 0600);
+   dbmopen (%HDB, "$srcdb$config{'dbmopen_ext'}", 0600);
 
    if ($op eq "move" || $op eq "copy") {
       filelock("$dstdb$config{'dbm_ext'}", LOCK_EX);
-      dbmopen (%HDB2, "$dstdb", 0600);
+      dbmopen (%HDB2, "$dstdb$config{'dbmopen_ext'}", 0600);
    }
 
    $blockstart=$blockend=$writepointer=0;
@@ -1174,8 +1183,14 @@ sub operate_message_with_ids {
       $messagevalid=1;
 
       seek($folderhandle, $attr[$_OFFSET], 0);
-      read($folderhandle, $buff, 5);
-      $messagevalid=0 if ($buff!~/^From /);
+      if ($attr[$_OFFSET] == 0) {
+         $messagevalid=1;
+      } elsif ($attr[$_SIZE]<=5) {
+         $messagevalid=0;
+      } else {
+         read($folderhandle, $buff, 5);
+         $messagevalid=0 if ($buff!~/^From /);
+      }
 
       if ($messageids =~ /^\Q$allmessageids[$i]\E$/m && $messagevalid) { # msg to be operated
          $counted++;
@@ -1274,7 +1289,7 @@ sub delete_message_by_age {
    @allmessageids=get_messageids_sorted_by_offset($headerdb);
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
-   dbmopen (%HDB, $headerdb, 0600);
+   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
    foreach (@allmessageids) {
       my @attr = split(/@@@/, $HDB{$_});
       push(@agedids, $_) if (dateserial2age($attr[$_DATE])>=$age);
@@ -1295,7 +1310,7 @@ sub move_oldmsg_from_folder {
    my @messageids=();
 
    filelock("$srcdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen (%HDB, $srcdb, undef);
+   dbmopen (%HDB, "$srcdb$config{'dbmopen_ext'}", undef);
 
    # if oldmsg == internal msg or 0, then do not read ids
    if ( $HDB{'ALLMESSAGES'}-$HDB{'NEWMESSAGES'} > $HDB{'INTERNALMESSAGES'} ) {
@@ -1335,7 +1350,7 @@ sub rebuild_message_with_partialid {
 
    # find all partial msgids
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen (%HDB, $headerdb, undef);
+   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    @messageids=keys %HDB;
    foreach my $id (@messageids) {
       next if ( $id eq 'METAINFO' 
@@ -1346,6 +1361,7 @@ sub rebuild_message_with_partialid {
 
       my @attr=split( /@@@/, $HDB{$id} );
       next if ($attr[$_CONTENT_TYPE] !~ /^message\/partial/i );
+
       $attr[$_CONTENT_TYPE] =~ /;\s*id="(.+?)";?/i;
       next if ($partialid ne $1);
 
@@ -1370,7 +1386,8 @@ sub rebuild_message_with_partialid {
       }
    }
 
-   my ($tmpfile, $tmpdb)=get_folderfile_headerdb($user, "_rebuild_tmp_$$");
+   my $tmpfile="/tmp/rebuild_tmp_$$";
+   my $tmpdb="/tmp/.rebuild_tmp_$$";
    ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);
    ($tmpdb =~ /^(.+)$/) && ($tmpdb = $1);
 
@@ -1421,22 +1438,12 @@ sub rebuild_message_with_partialid {
       return(-4);
    }
 
-   my ($trashfile, $trashdb)=get_folderfile_headerdb($user, "mail-trash");
-   ($trashfile =~ /^(.+)$/) && ($trashfile = $1);
-   ($trashdb =~ /^(.+)$/) && ($trashdb = $1);
-
-   # move partial msgs to trash folder
-   if ($folderfile ne $trashfile) {
-      filelock("$trashfile", LOCK_EX);
-      my $moved=operate_message_with_ids("move", \@partialmsgids, 
-				$folderfile, $headerdb, $trashfile, $trashdb);
-      filelock("$trashfile", LOCK_UN);
-   }
-
    my $moved=operate_message_with_ids("move", \@rebuildmsgids, 
 				$tmpfile, $tmpdb, $folderfile, $headerdb);
+
    unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
-   return(0, $rebuildmsgids[0]);
+
+   return(0, $rebuildmsgids[0], @partialmsgids);
 }
 
 ##################### END REBUILD_MESSAGE_WITH_PARTIALID #################
@@ -1472,7 +1479,9 @@ sub parse_rfc822block {
       my $boundarylen;
       my ($bodystart, $boundarystart, $nextboundarystart, $attblockstart);
 
-      $boundary =~ s/.*?boundary\s?=\s?"?([^\s"]+)[\s"]?.*$/$1/i;
+      $boundary =~ s/.*?boundary\s?=\s?"([^"]+)".*$/$1/i ||
+      $boundary =~ s/.*?boundary\s?=\s?([^\s]+)\s?.*$/$1/i;
+
       $boundary="--$boundary";
       $boundarylen=length($boundary);
 
@@ -1500,7 +1509,6 @@ sub parse_rfc822block {
          if ($nextboundarystart <= $attblockstart) {
             $nextboundarystart=index(${$r_block}, "$boundary--", $attblockstart);
          }
-
          if ($nextboundarystart > $attblockstart) {
             # normal attblock handling
             if ( $searchid eq "" || $searchid eq "all") {
@@ -1509,7 +1517,7 @@ sub parse_rfc822block {
             } elsif ($searchid eq "$nodeid-$i" || $searchid=~/^$nodeid-$i-/) {
                my $r_attachments2=parse_attblock($r_block, $attblockstart, $nextboundarystart-$attblockstart, $subtype, $boundary, "$nodeid-$i", $searchid);
                push(@attachments, @{$r_attachments2});
-               last;	# attblock after this is not he one to look for...
+               last;	# attblock after this is not the one to look for...
             }
             $boundarystart=$nextboundarystart;
             $attblockstart=$boundarystart+$boundarylen;
@@ -1517,8 +1525,11 @@ sub parse_rfc822block {
             # abnormal attblock, last one?
             if ( $searchid eq "" || $searchid eq "all" || 
                  $searchid eq "$nodeid-$i" || $searchid=~/^$nodeid-$i-/ ) {
-               my $r_attachments2=parse_attblock($r_block, $attblockstart, length(${$r_block})-$attblockstart ,$subtype, $boundary, "$nodeid-$i", $searchid);
-               push(@attachments, @{$r_attachments2});
+               my $left=length(${$r_block})-$attblockstart;
+               if ($left>0) {
+                  my $r_attachments2=parse_attblock($r_block, $attblockstart, $left ,$subtype, $boundary, "$nodeid-$i", $searchid);
+                  push(@attachments, @{$r_attachments2});
+               }
             }
             last;
          }
@@ -1595,7 +1606,8 @@ sub parse_rfc822block {
          # null searchid means CGI is in returning html code or in context searching
          # thus content of an non-text based attachment is no need to be returned
          my $bodylength=length(${$r_block})-($headerlen+2);
-         push(@attachments, make_attachment("","", "",\"snipped...",$bodylength,
+         my $fakeddata="snipped...";
+         push(@attachments, make_attachment("","", "",\$fakeddata,$bodylength,
 					$encoding,$contenttype, "","","",$description, $nodeid) );
       }
       return($header, " ", \@attachments);
@@ -1608,7 +1620,7 @@ sub parse_attblock {
    my ($r_buff, $attblockstart, $attblocklen, $subtype, $boundary, $nodeid, $searchid)=@_;
    my @attachments=();
    my ($attheader, $attcontent, $attencoding, $attcontenttype, 
-	$attdisposition, $attid, $attlocation);
+	$attdisposition, $attid, $attlocation, $attdescription);
    my $attheaderlen;
    if (/^\-\-\n/) {	# return empty array
       return(\@attachments) 
@@ -1658,6 +1670,9 @@ sub parse_attblock {
       my ($boundarystart, $nextboundarystart, $subattblockstart);
       my $subattblock="";
 
+      $boundary =~ s/.*?boundary\s?=\s?"([^"]+)".*$/$1/i ||
+      $boundary =~ s/.*?boundary\s?=\s?([^\s]+)\s?.*$/$1/i;
+
       $boundary =~ s/.*?boundary\s?=\s?"?([^\s"]+)[\s"]?.*$/$1/i;
       $boundary="--$boundary";
       $boundarylen=length($boundary);
@@ -1710,8 +1725,11 @@ sub parse_attblock {
             # abnormal attblock, last one?
             if ( $searchid eq "" || $searchid eq "all" || 
                  $searchid eq "$nodeid-$i" || $searchid=~/^$nodeid-$i-/ ) {
-               my $r_attachments2=parse_attblock($r_buff, $subattblockstart, $attblocklen-$subattblockstart ,$subtype, $boundary, "$nodeid-$i", $searchid);
-               push(@attachments, @{$r_attachments2});
+               my $left=$attblocklen-$subattblockstart;
+               if ($left>0) {
+                  my $r_attachments2=parse_attblock($r_buff, $subattblockstart, $left ,$subtype, $boundary, "$nodeid-$i", $searchid);
+                  push(@attachments, @{$r_attachments2});
+               }
             }
             last;
          }
@@ -1737,6 +1755,13 @@ sub parse_attblock {
    } elsif ($attcontenttype =~ /^message/i ) {
       if ( $searchid eq "" || $searchid eq "all" || $searchid=~/^$nodeid/ ) {
          $attcontent=substr(${$r_buff}, $attblockstart+$attheaderlen+2, $attblocklen-($attheaderlen+2));
+         if ( $attencoding =~ /^quoted-printable/i) {
+            $attcontent = decode_qp($attcontent);
+         } elsif ($attencoding =~ /^base64/i) {
+            $attcontent = decode_base64($attcontent);
+         } elsif ($attencoding =~ /^x-uuencode/i) {
+            $attcontent = uudecode($attcontent);
+         }
          my ($header2, $body2, $r_attachments2)=parse_rfc822block(\$attcontent, "$nodeid-0", $searchid);
          if ( $searchid eq "" || $searchid eq "all" || $searchid eq $nodeid ) {
             $header2 = decode_mimewords($header2);
@@ -1770,7 +1795,8 @@ sub parse_attblock {
          # null searchid means CGI is in returning html code or in context searching
          # thus content of an non-text based attachment is no need to be returned
          my $attcontentlength=$attblocklen-($attheaderlen+2);
-         push(@attachments, make_attachment($subtype,$boundary, $attheader,\"snipped...",$attcontentlength,
+         my $fakeddata="snipped...";
+         push(@attachments, make_attachment($subtype,$boundary, $attheader,\$fakeddata,$attcontentlength,
 		$attencoding,$attcontenttype, $attdisposition,$attid,$attlocation,$attdescription, $nodeid) );
       }
 
@@ -1790,15 +1816,9 @@ sub parse_uuencode_body {
       if ( $searchid eq "" || $searchid eq "all" || $searchid eq "$nodeid-$i" ) {
          my ($uumode, $uufilename, $uubody) = ($1, $2, $3);
          my $uutype;
-         if ($uufilename=~/\.doc$/i) {
-            $uutype="application/msword";
-         } elsif ($uufilename=~/\.ppt$/i) {
-            $uutype="application/x-mspowerpoint";
-         } elsif ($uufilename=~/\.xls$/i) {
-            $uutype="application/x-msexcel";
-         } else {
-            $uutype="application/octet-stream";
-         }
+
+         $uufilename=~/\.([\w\d]+)$/;
+         $uutype=ext2contenttype($1);
 
          # convert and inline uuencode block into an base64 encoded attachment
          my $uuheader=qq|Content-Type: $uutype;\n|.
@@ -1878,6 +1898,15 @@ sub make_attachment {
 
    $attdisposition =~ s/^(.+);.*/$1/g;
 
+   # guess a better contenttype
+   if ( $attcontenttype =~ m!(\Qapplication/octet-stream\E)!i ||
+        $attcontenttype =~ m!(\Qvideo/mpg\E)!i ) {
+      my $oldtype=$1;
+      $attfilename=~ /\.([\w\d]*)$/;
+      my $newtype=ext2contenttype($1);
+      $attcontenttype=~ s!$oldtype!$newtype!i;
+   }
+
    # the 2 attr are coming from parent block
    $temphash{subtype} = $subtype;
    $temphash{boundary} = $boundary;
@@ -1927,6 +1956,31 @@ sub contenttype2ext {
    return("js")  if ($ext =~ /javascript/i);
    return("bin");
 }
+
+sub ext2contenttype {
+   my $ext=$_[0];
+
+   return("application/msword") 	if ($ext =~ /doc$/i);
+   return("application/x-mspowerpoint") if ($ext =~ /ppt$/i);
+   return("application/x-msexcel") 	if ($ext =~ /xls$/i);
+   return("video/mpeg")			if ($ext =~ /(mpeg|mpg|mp2|mpe)$/i);
+   return("video/x-msvideo")		if ($ext =~ /(avi|wav|dl|fli)$/i);
+   return("video/quicktime")		if ($ext =~ /(mov|qt)$/i);
+   return("audio/mpeg")			if ($ext =~ /mp3$/i);
+   return("text/xml")			if ($ext =~ /xml$/i);
+   return("text/html")			if ($ext =~ /(html|htm)$/i);
+   return("text/plain")			if ($ext =~ /(txt|text|c|h|cpp|cc|asm|pas|f77|lst|sh|pl)$/i);
+   return("application/pdf")		if ($ext =~ /pdf$/i);
+   return("application/postscript")	if ($ext =~ /(ps|eps|ai)$/i);
+   return("application/mac-binhex40")	if ($ext =~ /hqx$/i);
+   return("application/x-javascript")	if ($ext =~ /js$/i);
+   return("application/x-msvisio")	if ($ext =~ /visio$/i);
+   return("application/x-vcard")	if ($ext =~ /vcf$/i);
+   return("application/x-shockwave-flash")	if ($ext =~ /swf$/i);
+   return("application/x-zip-compressed")	if ($ext =~ /zip$/i);
+
+   return("application/octet-stream");
+}
 ####################### END PARSE_.... related ###########################
 
 #################### SEARCH_MESSAGES_FOR_KEYWORD ###########################
@@ -1939,7 +1993,7 @@ sub search_info_messages_for_keyword {
    my %found=();
 
    filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-   dbmopen (%HDB, $headerdb, undef);
+   dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    $metainfo=$HDB{'METAINFO'};
    dbmclose(%HDB);
    filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
@@ -1959,7 +2013,7 @@ sub search_info_messages_for_keyword {
    if ( $cache_metainfo ne $metainfo || $cache_headerdb ne $headerdb ||
         $cache_keyword ne $keyword || $cache_searchtype ne $searchtype ||
         $cache_ignore_internal ne $ignore_internal ) {
-      ($cachefile =~ /^(.+)$/) && ($cachefile = $1);		# bypass taint check
+      ($cachefile =~ /^(.+)$/) && ($cachefile = $1);		# untaint ...
       open(CACHE, ">$cachefile");
       print CACHE $metainfo, "\n";
       print CACHE $headerdb, "\n";
@@ -1970,7 +2024,7 @@ sub search_info_messages_for_keyword {
       @messageids=get_messageids_sorted_by_offset($headerdb, $folderhandle);
 
       filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%HDB, $headerdb, undef);
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
 
       foreach $messageid (@messageids) {
          my (@attr, $block, $header, $body, $r_attachments) ;
@@ -2112,8 +2166,9 @@ sub search_info_messages_for_keyword {
 #################### END SEARCH_MESSAGES_FOR_KEYWORD ######################
 
 ######################## HTML related ##############################
-# it is suggest calling these following routine in the following order:
-# html4base, html4link, html4js, html4attachment, html4mailto, html2table
+# it is suggested calling these following routine in the following order:
+# html4nobase, html4link, html4disablejs, html4disableembcgi, 
+# html4attachment, html4mailto, html2table
 
 # since this routine deals with base directive, 
 # it must be called first before other html...routines when converting html
@@ -2127,23 +2182,13 @@ sub html4nobase {
 
    $html =~ s#\<base\s+([^\<\>]*?)\>##gi;
    if ( ($urlbase ne "") && ($urlbase !~ /^file:/) ) {
-      $html =~ s#(\<a\s+href=\s*"?)#$1$urlbase#gi;
-      $html =~ s#(src\s*=\s*"?)#$1$urlbase#gi;
-      $html =~ s#(background\s*=\s*"?)#$1$urlbase#gi;
-
-      # restore links that should be chnaged by base directive
-      $html =~ s#\Q$urlbase\E(http://)#$1#gi;
-      $html =~ s#\Q$urlbase\E(https://)#$1#gi;
-      $html =~ s#\Q$urlbase\E(ftp://)#$1#gi;
-      $html =~ s#\Q$urlbase\E(cid:)#$1#gi;
-      $html =~ s#\Q$urlbase\E(mailto:)#$1#gi;
+      $html =~ s#(\<a\s+href|background|src|method|action)(=\s*"?)#$1$2$urlbase#gi;
+      # recover links that should not be changed by base directive
+      $html =~ s#\Q$urlbase\E(http://|https://|ftp://|cid:|mailto:)#$1#gi;
    }
-
    return($html);
 }
 
-# this routine disables the javascript in a html page
-# to avoid user being hijacked by some eval programs
 my @jsevents=('onAbort', 'onBlur', 'onChange', 'onClick', 'onDblClick', 
               'onDragDrop', 'onError', 'onFocus', 'onKeyDown', 'onKeyPress', 
               'onKeyUp', 'onLoad', 'onMouseDown', 'onMouseMove', 'onMouseOut',
@@ -2154,23 +2199,25 @@ my @jsevents=('onAbort', 'onBlur', 'onChange', 'onClick', 'onDblClick',
 # so clicking on it will open a new window
 sub html4link {
    my $html=$_[0];
-   $html=~s/(<a [^\<\>]*?>)/_link_target_blank($1)/igems;
+   $html=~s/(<a\s+[^\<\>]*?>)/_link_target_blank($1)/igems;
    return($html);
 }
 
 sub _link_target_blank {
    my $link=$_[0];
-   foreach my $event (@jsevents) {
-      return($link) if ($link =~ /$event/i);
-   }
+#   foreach my $event (@jsevents) {
+#      return($link) if ($link =~ /$event/i);
+#   }
    if ($link =~ /target=/i ||
        $link =~ /javascript:/i ) {
       return($link);
    }
-   $link=~s/<a ([^\<\>]*?)>/<a $1 target=_blank>/;
+   $link=~s/<a\s+([^\<\>]*?)>/<a $1 target=_blank>/is;
    return($link);
 }
 
+# this routine disables the javascript in a html message
+# to avoid user being hijacked by some eval programs
 sub html4disablejs {
    my $html=$_[0];
    my $event;
@@ -2187,6 +2234,25 @@ sub html4disablejs {
    return($html);
 }
 
+# this routine disables the embedded CGI in a html message
+# to avoid user email addresses being confirmed by spammer through embedded CGIs
+sub html4disableembcgi {
+   my $html=$_[0];
+   $html=~s!(src|background)\s*=\s*("?https?://[\w\.\-]+?/?[^\s<>]*[\w/])([\b|\n| ]*)!_clean_embcgi($1,$2,$3)!egis;
+   return($html);
+}
+
+sub _clean_embcgi {
+   my ($type, $url, $end)=@_;
+
+   if ($url=~/\?/s && $url !~ /\Q$ENV{'HTTP_HOST'}\E/is) { # non local CGI found
+      $url=~s/["']//g;
+      return("alt='Embedded CGI removed by $config{'name'}.\n$url'".$end);
+   } else {
+      return("$type=$url".$end);
+   }
+}
+
 # this routine is used to resolve crossreference inside attachments
 # by converting them to request attachment from openwebmail cgi
 sub html4attachments {
@@ -2247,8 +2313,8 @@ sub html2text {
    my $t=$_[0];
 
    $t=~s!\s+! !g;
-   $t=~s|<style>.*?</style>||ig;
-   $t=~s|<script>.*?</script>||ig;
+   $t=~s|<style>.*?</style>||isg;
+   $t=~s|<script>.*?</script>||isg;
 
    $t=~s!<title[^\<\>]*?>!\n\n!ig;
    $t=~s!</title>!\n\n!ig;
@@ -2266,7 +2332,7 @@ sub html2text {
    $t=~s!<td[^\<\>]*?>! !ig;
    $t=~s!</td>! !ig;
 
-   $t=~s!<--.*?-->!!ig;
+   $t=~s!<--.*?-->!!isg;
 
    $t=~s!<[^\<\>]*?>!!gsm;
 
@@ -2275,6 +2341,7 @@ sub html2text {
    $t=~s!&gt;!>!g;
    $t=~s!&amp;!&!g;
    $t=~s!&quot;!\"!g;
+   $t=~s!&#8364;!!g;	# Euro symbo 
 
    $t=~s!\n\n\s+!\n\n!g;
 
@@ -2284,18 +2351,25 @@ sub html2text {
 sub text2html {
    my $t=$_[0];
 
-   $t=~s/&/&amp;/g;
-   $t=~s/\"/&quot;/g;
-   $t=~s/</&lt;/g;
-   $t=~s/>/&gt;/g;
-   $t=~s/\n/<BR>\n/g;
+   $t=~s/&/ESCAPE_AMP/g;
+
+   $t=~s!!&#8364;!g;	# Euro symbo 
+   $t=~s/\"/ &quot;/g;
+   $t=~s/</ &lt;/g;
+   $t=~s/>/ &gt;/g;
+
    $t=~s/ {2}/ &nbsp;/g;
    $t=~s/\t/ &nbsp;&nbsp;&nbsp;&nbsp;/g;
-         
+   $t=~s/\n/<BR>\n/g;
+
    foreach (qw(http https ftp nntp news gopher telnet)) {
-      $t=~s!($_://[\w\.\-]+?/?[^\s<>]*[\w/])([\b|\n| ]*)!<a href="$1" target="_blank">$1</A>$2!gs;
+      $t=~s!($_://[\w\d\-\.]+?/?[^\s<>]*[\w/])([\b|\n| ]*)!<a href="$1" target="_blank">$1</A>$2!gs;
    }
-   $t=~s!([\b|\n| ]+)(www\.[\w\-\.]+\.[\w\-]{2,3})([\b|\n| ]*)!$1<a href="http://$2" target="_blank">$2</a>$3!gs;
+   $t=~s!([\b|\n| ]+)(www\.[\w\d\-\.]+\.[\w\d\-]{2,3})([\b|\n| ]*)!$1<a href="http://$2" target="_blank">$2</a>$3!gs;
+
+   # remove the blank inserted just now
+   $t=~s/ (&quot;|&lt;|&gt;)/$1/g;
+   $t=~s/ESCAPE_AMP/&amp;/g;
 
    return($t);
 }
