@@ -3,9 +3,26 @@
 # openwebmail.pl, openwebmail-main.pl, openwebmail-prefs.pl, 
 # spellcheck and checkmail.pl
 #
+use strict;
+use Fcntl qw(:DEFAULT :flock);
+
+use vars qw(%languagenames);
+
+# extern vars 
+# defined in caller openwebmail-xxx.pl
+use vars qw($SCRIPT_DIR);
+use vars qw(%config %config_raw);
+use vars qw($thissession);
+use vars qw($loginname $domain $user $userrealname $uuid $ugid $homedir);
+use vars qw(%prefs %style);
+use vars qw($folderdir @validfolders $folderusage);
+use vars qw($folder $printfolder $escapedfolder);
+use vars qw($sort $searchtype $keyword);
+use vars qw($lang_charset %lang_folders %lang_text %lang_err);	# defined in lang/xy
 
 # languagenames - The abbreviation of the languages and related names
 %languagenames = (
+                 'bg'		=> 'Bulgarian',
                  'ca'           => 'Catalan',
                  'cs'           => 'Czech',
                  'da'           => 'Danish',
@@ -15,6 +32,7 @@
                  'fi'           => 'Finnish',
                  'fr'           => 'French',
                  'hu'           => 'Hungarian',
+                 'id'		=> 'Indonesian',
                  'it'           => 'Italiano',
                  'kr'           => 'Korean',
                  'lt'           => 'Lithuanian',
@@ -27,15 +45,120 @@
                  'ru'           => 'Russian',
                  'sk'           => 'Slovak',
                  'sv'           => 'Swedish',			# Svenska
+                 'tr'           => 'Turkish',
+                 'uk'           => 'Ukrainian',
                  'zh_CN.GB2312' => 'Chinese ( Simplified )',
                  'zh_TW.Big5'   => 'Chinese ( Traditional )'
                  );
 
+###################### OPENWEBMAIL_INIT ###################
+# init routine to set globals, switch euid
+sub openwebmail_init {
+   readconf(\%config, \%config_raw, "$SCRIPT_DIR/etc/openwebmail.conf.default");
+   readconf(\%config, \%config_raw, "$SCRIPT_DIR/etc/openwebmail.conf") if (-f "$SCRIPT_DIR/etc/openwebmail.conf");
+   # setuid is required if mails is located in user's dir
+   if ( $>!=0 && ($config{'use_homedirspools'}||$config{'use_homedirfolders'}) ) {
+      print "Content-type: text/html\n\n'$0' must setuid to root"; exit 0;
+   }
+
+   if (! defined(param("sessionid")) ) {
+      sleep $config{'loginerrordelay'};	# delayed response
+      openwebmailerror("No user specified!");
+   }
+
+   $thissession = param("sessionid");
+
+   $loginname = $thissession || '';
+   $loginname =~ s/\-session\-0.*$//; # Grab loginname from sessionid
+
+   my $siteconf;
+   if ($loginname=~/\@(.+)$/) {
+       $siteconf="$config{'ow_etcdir'}/sites.conf/$1";
+   } else {
+       my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
+       $siteconf="$config{'ow_etcdir'}/sites.conf/$httphost";
+   }
+   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf"); 
+
+   if ($config{'smtpauth'}) {
+      readconf(\%config, \%config_raw, "$SCRIPT_DIR/etc/smtpauth.conf");
+      if ($config{'smtpauth_username'} eq "" || $config{'smtpauth_password'} eq "") {
+         openwebmailerror("Invalid username/password in $SCRIPT_DIR/etc/smtpauth.conf");
+      }
+   }
+
+   require $config{'auth_module'} or
+      openwebmailerror("Can't open authentication module $config{'auth_module'}");
+
+   ($loginname, $domain, $user, $userrealname, $uuid, $ugid, $homedir)
+	=get_domain_user_userinfo($loginname);
+   if ($user eq "") {
+      sleep $config{'loginerrordelay'};	# delayed response
+      openwebmailerror("User $loginname doesn't exist!");
+   }
+   
+   my $userconf="$config{'ow_etcdir'}/users.conf/$user";
+   $userconf .= "\@$domain" if ($config{'auth_withdomain'});
+   readconf(\%config, \%config_raw, "$userconf") if ( -f "$userconf"); 
+
+   # override auto guessing domainanmes if loginame has domain
+   if ($config_raw{'domainnames'} eq 'auto' && $loginname=~/\@(.+)$/) {
+      $config{'domainnames'}=[ $1 ];
+   }
+
+   if ( $config{'use_homedirspools'} || $config{'use_homedirfolders'} ) {
+      my $mailgid=getgrnam('mail');
+      set_euid_egid_umask($uuid, $mailgid, 0077);	
+      if ( $) != $mailgid) {	# egid must be mail since this is a mail program...
+         openwebmailerror("Set effective gid to mail($mailgid) failed!");
+      }
+   }
+
+   if ( $config{'use_homedirfolders'} ) {
+      $folderdir = "$homedir/$config{'homedirfolderdirname'}";
+   } else {
+      $folderdir = "$config{'ow_etcdir'}/users/$user";
+      $folderdir .= "\@$domain" if ($config{'auth_withdomain'});
+   }
+
+   ($user =~ /^(.+)$/) && ($user = $1);  # untaint ...
+   ($uuid =~ /^(.+)$/) && ($uuid = $1);
+   ($ugid =~ /^(.+)$/) && ($ugid = $1);
+   ($homedir =~ /^(.+)$/) && ($homedir = $1);
+   ($folderdir =~ /^(.+)$/) && ($folderdir = $1);
+
+   %prefs = %{&readprefs};
+   %style = %{&readstyle};
+
+   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
+   require "etc/lang/$prefs{'language'}";
+   $lang_charset ||= 'iso-8859-1';
+
+   getfolders(\@validfolders, \$folderusage);
+   if (param("folder")) {
+      my $isvalid = 0;
+      $folder = param("folder");
+      foreach my $checkfolder (@validfolders) {
+         if ($folder eq $checkfolder) {
+            $isvalid = 1;
+            last;
+         }
+      }
+      ($folder = 'INBOX') unless ( $isvalid );
+   } else {
+      $folder = "INBOX";
+   }
+
+   $printfolder = $lang_folders{$folder} || $folder || '';
+   $escapedfolder = escapeURL($folder);
+}
+#################### END OPENWEBMAIL_INIT ###################
+
 ####################### READCONF #######################
 # read openwebmail.conf into a hash
 # the hash is 'called by reference' since we want to do 'bypass taint' on it
 sub readconf {
-   my ($r_confighash, $configfile)=@_;
+   my ($r_config, $r_config_raw, $configfile)=@_;
 
    # read config
    open(CONFIG, $configfile) or
@@ -47,10 +170,8 @@ sub readconf {
       if ($blockmode) {
          if ( $line =~ m!</$key>! ) {
             $blockmode=0;
-            # resolv %var% but forward reference is not supported
-            ${$r_confighash}{$key} =~ s/\%([\w\d_]+)\%/${$r_confighash}{$1}/msg; 
          } else {
-            ${$r_confighash}{$key} .= "$line\n";
+            ${$r_config_raw}{$key} .= "$line\n";
          }
       } else { 
          $line=~s/#.*$//;
@@ -59,72 +180,100 @@ sub readconf {
 
          if ( $line =~ m!^<(.+)>$! ) {
             $key=$1; $key=~s/^\s+//; $key=~s/\s+$//;
-            ${$r_confighash}{$key}="";
+            ${$r_config_raw}{$key}="";
             $blockmode=1;
          } else {
             ($key, $value)=split(/\s+/, $line, 2);
             if ($key ne "" && $value ne "" ) {
-               # resolv %var% but forward reference is not supported
-               $value =~ s/\%([\w\d_]+)\%/${$r_confighash}{$1}/msg; 
-               ${$r_confighash}{$key}=$value; 
+               ${$r_config_raw}{$key}=$value; 
             }
          }
       }
    }
    close(CONFIG);
 
+   # copy config_raw to config
+   %{$r_config}=%{$r_config_raw};
+   # resolv %var% in hash config
+   foreach $key (keys %{$r_config}) {
+      for (my $i=0; $i<5; $i++) {
+        last if (${$r_config}{$key} !~ s/\%([\w\d_]+)\%/${$r_config}{$1}/msg);
+      }
+   }
+
    # processing yes/no
-   foreach $key ( 'use_hashedmailspools', 'use_homedirspools',
+   foreach $key ( 'smtpauth',
+		  'use_hashedmailspools', 'use_homedirspools',
                   'use_homedirfolders', 'use_dotlockfile', 
-                  'deliver_use_GMT', 'savedsuid_support',
+                  'auth_withdomain', 'deliver_use_GMT', 'savedsuid_support',
                   'refresh_after_login', 'enable_rootlogin',
-                  'enable_changepwd', 'enable_setfromemail', 
-                  'enable_autoreply', 'enable_pop3', 'enable_setforward',
-                  'autopop3_at_refresh', 'default_autopop3', 
+                  'enable_changepwd', 'enable_setfromemail', 'enable_about',
+                  'enable_autoreply', 'enable_setforward',
+                  'enable_pop3', 'delpop3mail_by_default',
+                  'getmail_from_pop3_authserver', 
+                  'default_autopop3', 
                   'default_reparagraphorigmsg',
                   'default_confirmmsgmovecopy', 'default_viewnextaftermsgmovecopy',
                   'default_moveoldmsgfrominbox', 'forced_moveoldmsgfrominbox',
                   'default_hideinternal', 'symboliclink_mbox',
                   'default_filter_fakedsmtp', 'default_filter_fakedexecontenttype',
-                  'default_disablejs', 'default_newmailsound', 
+                  'default_disablejs', 'default_disableembcgi', 
+                  'default_newmailsound', 
                   'default_usefixedfont', 'default_usesmileicon') {
-      if (${$r_confighash}{$key} =~ /yes/i || ${$r_confighash}{$key} == 1) {
-         ${$r_confighash}{$key}=1;
+      if (${$r_config}{$key} =~ /yes/i || ${$r_config}{$key} == 1) {
+         ${$r_config}{$key}=1;
       } else {
-         ${$r_confighash}{$key}=0;
+         ${$r_config}{$key}=0;
       }
    }
 
    # processing auto
-   if ( ${$r_confighash}{'domainnames'} eq 'auto' ) {
-      $value=`/bin/hostname`;
-      $value=~s/^\s+//; $value=~s/\s+$//;
-      ${$r_confighash}{'domainnames'}=$value;
+   if ( ${$r_config}{'domainnames'} eq 'auto' ) {
+      if ($ENV{'HTTP_HOST'}=~/[A-Za-z]\./) {
+         $value=$ENV{'HTTP_HOST'}; $value=~s/:\d+$//;	# remove port number
+      } else {
+         $value=`/bin/hostname`;
+         $value=~s/^\s+//; $value=~s/\s+$//;
+      }
+      ${$r_config}{'domainnames'}=$value;
    }
-   if ( ${$r_confighash}{'timeoffset'} eq 'auto' ) {
-      ${$r_confighash}{'timeoffset'}=gettimeoffset();
+   if ( ${$r_config}{'timeoffset'} eq 'auto' ) {
+      ${$r_config}{'timeoffset'}=gettimeoffset();
    }
 
    # processing list
-   foreach $key ('domainnames', 'spellcheck_dictionaries', 'disallowed_pop3servers') {
-      if (! defined(@{${$r_confighash}{$key}}) ) { # conv str to list
-         my @list=split(/\s*,\s*/, ${$r_confighash}{$key});
-         ${$r_confighash}{$key}=\@list;
-      }
+   foreach $key ('domainnames', 'spellcheck_dictionaries', 
+		 'allowed_serverdomain',
+		 'allowed_clientdomain', 'allowed_clientip', 
+                 'allowed_receiverdomain', 'disallowed_pop3servers',
+                 'default_fromemails' ) {
+      my $liststr=${$r_config}{$key}; $liststr=~s/\s//g;
+      my @list=split(/,/, $liststr);
+      ${$r_config}{$key}=\@list;
    }
 
    # processing none
-   if ( ${$r_confighash}{'default_bgurl'} eq 'none' ) {
-      $value="${$r_confighash}{'ow_htmlurl'}/images/backgrounds/Transparent.gif";
-      ${$r_confighash}{'default_bgurl'}=$value;
+   if ( ${$r_config}{'default_bgurl'} eq 'none'|| ${$r_config}{'default_bgurl'} eq '""' ) {
+      $value="${$r_config}{'ow_htmlurl'}/images/backgrounds/Transparent.gif";
+      ${$r_config}{'default_bgurl'}=$value;
+   }
+   foreach $key ( 'dbmopen_ext', 'default_realname' ) {
+      if ( ${$r_config}{$key} eq 'none' || ${$r_config}{$key} eq '""' ) {
+         ${$r_config}{$key}="";
+      }
    }
 
-   # bypass taint check for pathname defined in openwebmail.conf
-   foreach $key ( 'smtpserver', 'auth_module', 
-        'mailspooldir', 'homedirspoolname', 'homedirfolderdirname', 'dbm_ext',
-        'ow_cgidir', 'ow_htmldir','ow_etcdir', 'logfile', 'spellcheck',
-	'vacationinit', 'vacationpipe', 'g2b_converter', 'b2g_converter' ) {
-      (${$r_confighash}{$key} =~ /^(.+)$/) && (${$r_confighash}{$key}=$1);
+   # untaint pathname variable defined in openwebmail.conf
+   foreach $key ( 'smtpserver', 'auth_module', 'virtusertable',
+        'mailspooldir', 'homedirspoolname', 'homedirfolderdirname', 
+        'dbm_ext', 'dbmopen_ext', 
+	'ow_cgidir', 'ow_htmldir','ow_etcdir', 'logfile', 
+	'vacationinit', 'vacationpipe', 
+	'spellcheck', 'g2b_converter', 'b2g_converter' ) {
+      (${$r_config}{$key} =~ /^(.+)$/) && (${$r_config}{$key}=$1);
+   }
+   foreach my $domain ( @{${$r_config}{'domainnames'}} ) {
+      ($domain =~ /^(.+)$/) && ($domain=$1);
    }
 
    return(0);
@@ -134,20 +283,21 @@ sub readconf {
 ##################### VIRTUALUSER related ################
 sub update_virtusertable {
    my ($virdb, $virfile)=@_;
-   my (%DB, %DBR, %DBS, $metainfo);
+   my (%DB, %DBR, $metainfo);
+
+   ($virdb =~ /^(.+)$/) && ($virdb = $1);		# untaint ...
 
    if (! -e $virfile) {
       unlink("$virdb$config{'dbm_ext'}") if (-e "$virdb$config{'dbm_ext'}");
-      unlink("$virdb.short$config{'dbm_ext'}") if (-e "$virdb.short$config{'dbm_ext'}");
+      unlink("$virdb.rev$config{'dbm_ext'}") if (-e "$virdb.rev$config{'dbm_ext'}");
       return;
    }
 
-   ($virdb =~ /^(.+)$/) && ($virdb = $1);		# bypass taint check
    if ( -e "$virdb$config{'dbm_ext'}" ) {
       my ($metainfo);
 
       filelock("$virdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%DB, $virdb, undef);
+      dbmopen (%DB, "$virdb$config{'dbmopen_ext'}", undef);
       $metainfo=$DB{'METAINFO'};
       dbmclose(%DB);
       filelock("$virdb$config{'dbm_ext'}", LOCK_UN);
@@ -155,29 +305,25 @@ sub update_virtusertable {
       return if ( $metainfo eq metainfo($virfile) );
    } 
 
-   writelog("update virtusertable.db");
+   writelog("update $virdb");
 
    unlink("$virdb$config{'dbm_ext'}",
-          "$virdb.rev$config{'dbm_ext'}",
-          "$virdb.short$config{'dbm_ext'}");
+          "$virdb.rev$config{'dbm_ext'}",);
 
-   dbmopen(%DB, $virdb, 0644);
+   dbmopen(%DB, "$virdb$config{'dbmopen_ext'}", 0644);
    filelock("$virdb$config{'dbm_ext'}", LOCK_EX);
    %DB=();	# ensure the virdb is empty
 
-   dbmopen(%DBR, "$virdb.rev", 0644);
+   dbmopen(%DBR, "$virdb.rev$config{'dbmopen_ext'}", 0644);
    filelock("$virdb.rev$config{'dbm_ext'}", LOCK_EX);
    %DBR=();
 
-   dbmopen(%DBS, "$virdb.short", 0644);
-   filelock("$virdb.short$config{'dbm_ext'}", LOCK_EX);
-   %DBS=();
-
    open (VIRT, $virfile);
    while (<VIRT>) {
       s/^\s+//;
       s/\s+$//;
       s/#.*$//;
+      s/(.*?)\@(.*?)%1/$1\@$2$1/;	# resolve %1 in virtusertable
 
       my ($vu, $u)=split(/[\s\t]+/);
       next if ($vu eq "" || $u eq "");
@@ -190,66 +336,25 @@ sub update_virtusertable {
       } else {
          $DBR{$u}.="$vu";
       }
-
-      if ($vu=~/^(.+)@.+/) {
-         my $shortname=$1;
-         if ( defined($DBS{$shortname}) ) {
-            $DBS{$shortname}.=",$vu";
-         } else {
-            $DBS{$shortname}=$vu;
-         }
-      }
    }
    close(VIRT);
 
    $DB{'METAINFO'}=metainfo($virfile);
 
-   filelock("$virdb.short$config{'dbm_ext'}", LOCK_UN);
-   dbmclose(%DBS);
-
    filelock("$virdb.rev$config{'dbm_ext'}", LOCK_UN);
    dbmclose(%DBR);
-
    filelock("$virdb$config{'dbm_ext'}", LOCK_UN);
    dbmclose(%DB);
    return;
 }
 
-sub get_virtualuser_by_user {
-   my ($user, $virdbr)=@_;
-   my (%DBR, $vu);
-
-   if ( -f "$virdbr$config{'dbm_ext'}" ) {
-      filelock("$virdbr$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%DBR, $virdbr, undef);
-      $vu=$DBR{$user};
-      dbmclose(%DBR);
-      filelock("$virdbr$config{'dbm_ext'}", LOCK_UN);
-   }
-   return($vu);
-}
-
-sub get_virtualuser_by_shortname {
-   my ($shortname, $virdbs)=@_;
-   my (%DBS, $vu);
-
-   if ( -f "$virdbs$config{'dbm_ext'}" ) {
-      filelock("$virdbs$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%DBS, $virdbs, undef);
-      $vu=$DBS{$shortname};
-      dbmclose(%DBS);
-      filelock("$virdbs$config{'dbm_ext'}", LOCK_UN);
-   }
-   return($vu);
-}
-
 sub get_user_by_virtualuser {
    my ($vu, $virdb)=@_;
    my (%DB, $u);
 
    if ( -f "$virdb$config{'dbm_ext'}" ) {
       filelock("$virdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%DB, $virdb, undef);
+      dbmopen (%DB, "$virdb$config{'dbmopen_ext'}", undef);
       $u=$DB{$vu};
       dbmclose(%DB);
       filelock("$virdb$config{'dbm_ext'}", LOCK_UN);
@@ -257,91 +362,121 @@ sub get_user_by_virtualuser {
    return($u);
 }
 
-sub get_virtualuser_user_userinfo {
+sub get_virtualuser_by_user {
+   my ($user, $virdbr)=@_;
+   my (%DBR, $vu);
+
+   if ( -f "$virdbr$config{'dbm_ext'}" ) {
+      filelock("$virdbr$config{'dbm_ext'}", LOCK_SH);
+      dbmopen (%DBR, "$virdbr$config{'dbmopen_ext'}", undef);
+      $vu=$DBR{$user};
+      dbmclose(%DBR);
+      filelock("$virdbr$config{'dbm_ext'}", LOCK_UN);
+   }
+   return($vu);
+}
+
+sub get_domain_user_userinfo {
    my $loginname=$_[0];
-   my ($virtualuser, $user, $realname, $uid, $gid, $homedir);
+   my ($domain, $user, $realname, $uid, $gid, $homedir);
+   my ($l, $u);
 
-   my $default_realname=$loginname; 
-   $default_realname=~s/\@.*^//;
+   if ($loginname=~/^(.*)\@(.*)$/) {
+      ($user, $domain)=($1, $2);
+   } else {
+       my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
+      ($user, $domain)=($loginname, $httphost);
+   }
+   my $default_realname=$user;
 
-   # loginname is a real user and not mappped by any virtualuser
-   $virtualuser=get_virtualuser_by_user($loginname, "$config{'ow_etcdir'}/virtusertable.rev");
-   if ($virtualuser eq "") {    # loginname is a real userid (uuu)
-      ($realname, $uid, $gid, $homedir)=get_userinfo($loginname);
-      if ($uid ne "") {
-         return("", $loginname, $realname||$default_realname, $uid, $gid, $homedir);
+   my $virtname=$config{'virtusertable'}; 
+   $virtname=~s!/!.!g; $virtname=~s/^\.+//;
+
+   $l="$user\@$domain";
+   $u=get_user_by_virtualuser($l, "$config{'ow_etcdir'}/$virtname");
+   if ($u eq "") {
+      my $d=$domain;      
+      if ($d=~s/^(mail|webmail|www|web)\.//) {
+         $l="$user\@$d";
+         $u=get_user_by_virtualuser($l, "$config{'ow_etcdir'}/$virtname");
+      } else {
+         $l="$user\@mail.$d";
+         $u=get_user_by_virtualuser($l, "$config{'ow_etcdir'}/$virtname");
+      }
+   }
+   if ($u eq "" && $loginname!~/\@/) {
+      $l=$user;
+      $u=get_user_by_virtualuser($l, "$config{'ow_etcdir'}/$virtname");
+   }
+   
+   if ($u ne "") {
+      $loginname=$l;
+      if ($u=~/^(.*)\@(.*)$/) {
+         ($user, $domain)=($1, $2);
+      } else {
+         my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
+         ($user, $domain)=($u, $httphost);
       }
    }
 
-   # loginname@HTTP_HOST is a virtualuser (uuu)
-   if ($loginname !~ /\@/) {
-      my $domain=$ENV{'HTTP_HOST'};
-      $user=get_user_by_virtualuser($loginname.'@'.$domain, "$config{'ow_etcdir'}/virtusertable");
-      if ($user eq "" && $domain=~s/^mail\.//) {
-         $user=get_user_by_virtualuser($loginname.'@'.$domain, "$config{'ow_etcdir'}/virtusertable");
-      }
-      if ($user ne "") {
-         ($realname, $uid, $gid, $homedir)=get_userinfo($user);
-         if ($uid ne "") {
-            return($loginname.'@'.$domain, $user, $realname||$default_realname, $uid, $gid, $homedir);
-         }
-      }
-   }
-
-   # loginname is a virtualuser (uuu or uuu@hhh)
-   $user=get_user_by_virtualuser($loginname, "$config{'ow_etcdir'}/virtusertable");
-   if ($user ne "") {
+   if ($config{'auth_withdomain'}) {
+      ($realname, $uid, $gid, $homedir)=get_userinfo("$user\@$domain");
+   } else {
       ($realname, $uid, $gid, $homedir)=get_userinfo($user);
-      if ($uid ne "") {
-         return($loginname, $user, $realname||$default_realname, $uid, $gid, $homedir);
-      }
    }
 
-   # loginname is the username part of a virtualuser (uuu)
-   $virtualuser=get_virtualuser_by_shortname($loginname, "$config{'ow_etcdir'}/virtusertable.short");
-   # and this username appears only in this virtualuser
-   if ($virtualuser ne "" && $virtualuser !~ /,/) {	
-      $user=get_user_by_virtualuser($virtualuser, "$config{'ow_etcdir'}/virtusertable");
-      if ($user ne "") {
-         ($realname, $uid, $gid, $homedir)=get_userinfo($user);
-         if ($uid ne "") {
-            return($virtualuser, $user, $realname||$default_realname, $uid, $gid, $homedir);
-         }
-      }
+   if ($config{'default_realname'} ne 'auto') {
+      $realname=$config{'default_realname'} 
+   } else {
+      $realname=$default_realname if ($realname eq "");
    }
 
-   # user not found
-   return("", "", "", "", "", "");
+   if ($uid ne "") {
+      return($loginname, $domain, $user, $realname, $uid, $gid, $homedir);
+   } else {
+      return("", "", "", "", "", "", "");
+   }
 }
 ##################### END VIRTUALUSER related ################
 
 ##################### GET_DEFAULTEMAILS, GET_USERFROM ################
 sub get_defaultemails {
-   my ($virtualuser, $user)=@_;
+   my ($loginname, $user)=@_;
    my @emails=();
 
-   if ($virtualuser ne "") {
-      if ($virtualuser =~ /\@/ ) {
-         push(@emails, $virtualuser);
-      } else {
-         foreach my $domain (@{$config{'domainnames'}}) {
-            push(@emails, "$virtualuser".'@'."$domain");
+   if ($config_raw{'default_fromemails'} eq "auto") {
+      my $virtname=$config{'virtusertable'}; 
+      $virtname=~s!/!.!g; $virtname=~s/^\.+//;
+      my $vu=get_virtualuser_by_user($user, "$config{'ow_etcdir'}/$virtname.rev");
+      if ($vu ne "") {
+         foreach my $name (str2list($vu)) {
+            if ($name=~/\@/) {
+               push(@emails, $name);
+            } else {
+               foreach my $host (@{$config{'domainnames'}}) {
+                  push(@emails, "$name\@$host");
+               }
+            }
+         }
+      } else {    
+         my $name=$loginname;
+         $name=$1 if ($loginname =~ /^(.+)\@/ );
+         foreach my $host (@{$config{'domainnames'}}) {
+            push(@emails, "$name\@$host");
          }
       }
    } else {
-      foreach my $domain (@{$config{'domainnames'}}) {
-         push(@emails, "$user".'@'."$domain");
-      }
+      push(@emails, @{$config{'default_fromemails'}});
    }
    return(@emails);
 }
 
 sub get_userfrom {
-   my ($virtualuser, $user, $realname, $frombook)=@_;
+   my ($loginname, $user, $realname, $frombook)=@_;
    my %from=();
 
    # get default fromemail
-   my @defaultemails=get_defaultemails($virtualuser, $user);
+   my @defaultemails=get_defaultemails($loginname, $user);
    foreach (@defaultemails) {
       $from{$_}=$realname;
    }
@@ -401,12 +536,14 @@ sub readprefs {
 
    # validate email with defaultemails if setfromemail is not allowed
    if (!$config{'enable_setfromemail'} || $prefshash{'email'} eq "") {
-      my @defaultemails=get_defaultemails($virtualuser, $user);
-      my $email;
-      foreach $email (@defaultemails) {
-         last if ($prefshash{'email'} eq $email);
+      my @defaultemails=get_defaultemails($loginname, $user);
+      my $valid=0;
+      foreach (@defaultemails) {
+         if ($prefshash{'email'} eq $_) {
+            $valid=1; last;
+         }
       }
-      if ($prefshash{'email'} ne $email) {
+      if (! $valid) {
          $prefshash{'email'}=$defaultemails[0];
       }
    }
@@ -414,17 +551,17 @@ sub readprefs {
    # get default value from config for err/undefined/empty prefs entries
 
    # entries disallowed to be empty
-   foreach $key ( 'language', 'dictionary', 'style', 'iconset', 'bgurl', 
-                  'sort', 'dateformat', 'headersperpage', 
-                  'editcolumns', 'editrows',
-                  'confirmmsgmovecopy', 'viewnextaftermsgmovecopy', 
-                  'reparagraphorigmsg', 'replywithorigmsg',
-                  'sendreceipt', 'moveoldmsgfrominbox',
-                  'filter_repeatlimit', 'filter_fakedsmtp', 
-                  'filter_fakedexecontenttype',
-                  'disablejs', 'hideinternal', 'newmailsound', 
-                  'usefixedfont', 'usesmileicon', 'autopop3',
-                  'trashreserveddays') {
+   foreach $key ( qw(language dictionary style iconset bgurl 
+                     sort dateformat headersperpage 
+                     editcolumns editrows
+                     confirmmsgmovecopy viewnextaftermsgmovecopy 
+                     reparagraphorigmsg replywithorigmsg
+                     sendreceipt moveoldmsgfrominbox
+                     filter_repeatlimit filter_fakedsmtp 
+                     filter_fakedexecontenttype
+                     disablejs disableembcgi hideinternal newmailsound 
+                     usefixedfont usesmileicon autopop3
+                     trashreserveddays sessiontimeout) ) {
       if ( !defined($prefshash{$key}) || $prefshash{$key} eq "" ) {
           $prefshash{$key}=$config{'default_'.$key};
       }
@@ -501,6 +638,7 @@ sub applystyle {
    $template =~ s/\@\@\@LOGO_URL\@\@\@/$config{'logo_url'}/g;
    $template =~ s/\@\@\@LOGO_LINK\@\@\@/$config{'logo_link'}/g;
    $template =~ s/\@\@\@PAGE_FOOTER\@\@\@/$config{'page_footer'}/g;
+   $template =~ s/\@\@\@SESSIONID\@\@\@/$thissession/g;
 
    $url="$config{'ow_cgiurl'}/openwebmail.pl";
    $template =~ s/\@\@\@SCRIPTURL\@\@\@/$url/g;
@@ -527,34 +665,86 @@ sub applystyle {
 }
 ################ END APPLYSTYLE ###########################
 
+################ READ/WRITE POP3BOOK ##############
+sub readpop3book {
+   my ($pop3book, $r_accounts) = @_;
+   my $i=0;
+
+   %{$r_accounts}=();
+
+   if ( -f "$pop3book" ) {
+      filelock($pop3book, LOCK_SH);
+      open (POP3BOOK,"$pop3book") or return(-1);
+      while (<POP3BOOK>) {
+      	 chomp($_);
+         my ($pop3host, $pop3user, $pop3passwd, $pop3lastid, $pop3del, $enable)
+							=split(/\@\@\@/, $_);
+         ${$r_accounts}{"$pop3host:$pop3user"} = "$pop3host\@\@\@$pop3user\@\@\@$pop3passwd\@\@\@$pop3lastid\@\@\@$pop3del\@\@\@$enable";
+         $i++;
+      }
+      close (POP3BOOK);
+      filelock($pop3book, LOCK_UN);
+   }
+   return($i);
+}
+
+sub writepop3book {
+   my ($pop3book, $r_accounts) = @_;
+
+   if (! -f "$pop3book" ) {
+      ($pop3book =~ /^(.+)$/) && ($pop3book = $1); # untaint ...
+      open (POP3BOOK,">$pop3book") or return (-1);
+      close(POP3BOOK);
+   }
+
+   filelock($pop3book, LOCK_EX);
+   open (POP3BOOK,">$pop3book") or return (-1);
+   foreach (values %{$r_accounts}) {
+     chomp($_);
+     print POP3BOOK $_ . "\n";
+   }
+   close (POP3BOOK);
+   filelock($pop3book, LOCK_UN);
+
+   return(0);
+}
+
+################ END GET/WRITEBACK POP3BOOK ##############
+
 ############## VERIFYSESSION ########################
 sub verifysession {
-   if ( (-M "$config{'ow_etcdir'}/sessions/$thissession") > $config{'sessiontimeout'}/60/24
+   if ( (-M "$config{'ow_etcdir'}/sessions/$thissession") > $prefs{'sessiontimeout'}/60/24
      || !(-e "$config{'ow_etcdir'}/sessions/$thissession")) {
+
+      my $delfile="$config{'ow_etcdir'}/sessions/$thissession";
+      ($delfile =~ /^(.+)$/) && ($delfile = $1);  # untaint ...
+      unlink ($delfile) if ( -e "$delfile");
+
       my $html = '';
       printheader();
+
       open (TIMEOUT, "$config{'ow_etcdir'}/templates/$prefs{'language'}/sessiontimeout.template") or
          openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/sessiontimeout.template!");
       while (<TIMEOUT>) {
          $html .= $_;
       }
       close (TIMEOUT);
-
       $html = applystyle($html);
-
       print $html;
 
-      printfooter();
+      printfooter(1);      
+
       writelog("session error - session $thissession timeout access attempt");
       writehistory("session error - session $thissession timeout access attempt");
       exit 0;
    }
    if ( -e "$config{'ow_etcdir'}/sessions/$thissession" ) {
       open (SESSION, "$config{'ow_etcdir'}/sessions/$thissession");
-      my $cookie = <SESSION>;
+      my $cookie = <SESSION>; chomp $cookie;
+      my $ip = <SESSION>; chomp $ip;
       close (SESSION);
-      chomp $cookie;
-      if ( cookie("$user-sessionid") ne $cookie) {
+      
+      if ( cookie("$user-sessionid") ne $cookie ) { # not check ip here due to dialup user
          writelog("session error - session $thissession hijack attempt!");
          writehistory("session error - session $thissession hijack attempt!");
          openwebmailerror("$lang_err{'inv_sessid'}");
@@ -564,11 +754,12 @@ sub verifysession {
    openwebmailerror("Session ID $lang_err{'has_illegal_chars'}") unless
       (($thissession =~ /^([\w\.\-\@]+)$/) && ($thissession = $1));
 
-   if ( !defined(param("refresh")) ) {
-      # extend the session lifetime only if this is not a auto-refresh
+   if ( !defined(param("refresh")) && param("action") ne "timeoutwarning" ) {
+      # extend the session lifetime only if not auto-refresh/timeoutwarning
       open (SESSION, "> $config{'ow_etcdir'}/sessions/$thissession") or
          openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions/$thissession!");
-      print SESSION cookie("$user-sessionid");
+      print SESSION cookie("$user-sessionid"), "\n";
+      print SESSION get_clientip(), "\n";
       close (SESSION);
    }
    return 1;
@@ -601,6 +792,9 @@ sub get_folderfile_headerdb {
       $headerdb="$folderdir/.$foldername";
    }
 
+   ($folderfile =~ /^(.+)$/) && ($folderfile = $1); # untaint ...
+   ($headerdb =~ /^(.+)$/) && ($headerdb = $1);
+
    return($folderfile, $headerdb);
 }
 
@@ -620,6 +814,8 @@ sub getfolders {
 
    while (defined($filename = readdir(FOLDERDIR))) {
 
+      ($filename =~ /^(.+)$/) && ($filename = $1);   # untaint data from readdir
+
       next if ( $filename eq "." || $filename eq ".." );
 
       # find internal file that are stale
@@ -633,14 +829,12 @@ sub getfolders {
              $1 ne 'filter.book' && 
              ! -f "$folderdir/$1" ) {
             # dbm or cache whose folder doesn't exist
-            ($filename =~ /^(.+)$/) && ($filename = $1); # bypass taint check
             push (@delfiles, "$folderdir/$filename");
             next;
          }
       # clean tmp file in msg rebuild
       } elsif ($filename=~/^_rebuild_tmp_\d+$/ ||
                $filename=~/^\._rebuild_tmp_\d+$/ ) {
-         ($filename =~ /^(.+)$/) && ($filename = $1); # bypass taint check
          push (@delfiles, "$folderdir/$filename");
          next;
       }
@@ -678,9 +872,8 @@ sub getfolders {
       $totalsize += ( -s "$spoolfile" ) || 0;
    } else {
       # create spool file with user uid, gid if it doesn't exist
-      ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1); # bypass taint check
-      open (F, ">>$spoolfile");
-      close(F);
+      ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1); # untaint ...
+      open (F, ">>$spoolfile"); close(F);
       chown ($uuid, $ugid, $spoolfile);
    }
 
@@ -710,15 +903,18 @@ sub getmessage {
    close($folderhandle);
    filelock($folderfile, LOCK_UN);
 
-   return \%message if (${$r_messageblock} eq "" );
+   if (${$r_messageblock} eq "") {	# msgid not found
+      writelog("db error - msg $messageid index missing in $folder");
+      writehistory("db error - msg $messageid index missing in $folder");
+      return \%message;
 
-   if (${$r_messageblock}!~/^From /) {	# db index inconsistance found
+   } elsif (${$r_messageblock}!~/^From / ) {	# db index inconsistance
       filelock($folderfile, LOCK_SH|LOCK_NB) or
          openwebmailerror("$lang_err{'couldnt_locksh'} $folderfile!");
 
       filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
       my %HDB;
-      dbmopen (%HDB, $headerdb, undef);
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       $HDB{'METAINFO'}="ERR";
       dbmclose(%HDB);
       filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
@@ -731,8 +927,10 @@ sub getmessage {
       close($folderhandle);
 
       filelock($folderfile, LOCK_UN);
-      writelog("db error - $folderfile index inconsistence fixed");
-      writehistory("db error - $folderfile index inconsistence fixed");
+      writelog("db error - msg $messageid index inconsistence in $folder");
+      writehistory("db error - msg $messageid index inconsistence in $folder");
+
+      return \%message if (${$r_messageblock} eq "" );
    }
 
    my ($currentheader, $currentbody, $r_currentattachments, $currentfrom, $currentdate,
@@ -1008,7 +1206,7 @@ sub writelog {
    return if ( ($config{'logfile'} eq 'no') || ( -l "$config{'logfile'}" ) );
 
    my $timestamp = localtime();
-   my $loggeduser = $virtualuser || $user || 'UNKNOWNUSER';
+   my $loggeduser = $loginname || 'UNKNOWNUSER';
    my $loggedip = get_clientip();
 
    open (LOGFILE,">>$config{'logfile'}") or 
@@ -1024,7 +1222,7 @@ sub writelog {
 sub writehistory {
    my ($logaction)=$_[0];
    my $timestamp = localtime();
-   my $loggeduser = $virtualuser || $user || 'UNKNOWNUSER';
+   my $loggeduser = $loginname || 'UNKNOWNUSER';
    my $loggedip = get_clientip();
 
    if ( -f "$folderdir/.history.log" ) {
@@ -1066,11 +1264,13 @@ sub writehistory {
 
 ################ END WRITEHISTORY ##################
 
-##################### PRINTHEADER #########################
+##################### PRINTHEADER/PRINTFOOTER #########################
+
 # $headerprinted is set to 1 once printheader is called and seto 0 until
 # printfooter is called. This variable is used to not print header again
 # in openwebmailerror
-local $headerprinted=0;
+my $headerprinted=0;
+
 sub printheader {
    my $cookie;
    my @headers=();
@@ -1108,41 +1308,56 @@ sub printheader {
       print $html;
    }
 }
-################### END PRINTHEADER #######################
 
-################### PRINTFOOTER ###########################
 sub printfooter {
+   my $mode=$_[0];
    my $html = '';
 
-   open (FOOTER, "$config{'ow_etcdir'}/templates/$prefs{'language'}/footer.template") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/footer.template!");
-   while (<FOOTER>) {
-      $html .= $_;
-   }
-   close (FOOTER);
-   
-   $html = applystyle($html);
-   
-   my $remainingseconds= 365*24*60*60;	# default timeout = 1 year
-   if ($thissession ne "") { 	# if this is a session
-      my $sessionage=(-M "$config{'ow_etcdir'}/sessions/$thissession"); 
-      if ($sessionage ne "") {	# if this session is valid
-         $remainingseconds= ($config{'sessiontimeout'}/60/24-$sessionage)
-                            *24*60*60 - (time()-$^T);
-      } 
+   $headerprinted = 0;
+
+   if ($mode==2) {	# read in timeout check jscript
+      open (FOOTER, "$config{'ow_etcdir'}/templates/$prefs{'language'}/footer.js.template") or
+         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/footer.js.template!");
+      while (<FOOTER>) {
+         $html .= $_;
+      }
+      close (FOOTER);
+
+      my $remainingseconds= 365*24*60*60;	# default timeout = 1 year
+      if ($thissession ne "") { 	# if this is a session
+         my $sessionage=(-M "$config{'ow_etcdir'}/sessions/$thissession"); 
+         if ($sessionage ne "") {	# if this session is valid
+            $remainingseconds= ($prefs{'sessiontimeout'}/60/24-$sessionage)
+				*24*60*60 - (time()-$^T);
+         } 
+      }
+      $html =~ s/\@\@\@REMAININGSECONDS\@\@\@/$remainingseconds/g;
    }
 
-   $html =~ s/\@\@\@USEREMAIL\@\@\@/$prefs{'email'}/g;
-   $html =~ s/\@\@\@REMAININGSECONDS\@\@\@/$remainingseconds/g;
+   if ($mode>=1) {	# print footer
+      open (FOOTER, "$config{'ow_etcdir'}/templates/$prefs{'language'}/footer.template") or
+         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/footer.template!");
+      while (<FOOTER>) {
+         $html .= $_;
+      }
+      close (FOOTER);
+   
+      $html = applystyle($html);
+      $html =~ s/\@\@\@USEREMAIL\@\@\@/$prefs{'email'}/g;
+   }
+
+   if ($mode==0) {	# null footer
+      $html=qq|</body></html>\n|;
+   }
 
    print $html;
-
-   $headerprinted = 0;
 }
-################# END PRINTFOOTER #########################
+################# END PRINTHEADER/PRINTFOOTER #########################
 
 ##################### OPENWEBMAILERROR ##########################
 sub openwebmailerror {
+   my $mailgid=getgrnam('mail');
+
    if (defined($ENV{'HTTP_HOST'})) {	# in CGI mode
       # load prefs if possible, or use default value
       my $background = $style{"background"}||"#FFFFFF";
@@ -1181,8 +1396,8 @@ sub openwebmailerror {
             qq|euid=$>, egid=$), mailgid=$mailgid\n|,
             qq|</font><BR>|,
             qq|</td></tr>|,
-            qq|</table>\n|;
-      print qq|<p align="center"><font size="-1"><BR>|,
+            qq|</table>\n|,
+            qq|<p align="center"><font size="-1"><BR>|,
             qq|$config{'page_footer'}<BR>|,
             qq|</FONT></FONT></P></BODY></HTML>|;
 
@@ -1207,7 +1422,7 @@ sub g2b {
    }
 
    my $tmpfile="/tmp/.openwebmail.tmp.$$";
-   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);   # bypass taint check
+   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);   # untaint ...
    open (TMP, ">$tmpfile");
    print TMP $_[0];		# orig gb
    close(TMP);
@@ -1236,7 +1451,7 @@ sub b2g {
    }
 
    my $tmpfile="/tmp/.openwebmail.tmp.$$";
-   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);   # bypass taint check
+   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);   # untaint ...
    open (TMP, ">$tmpfile");
    print TMP $_[0];		# orig big5
    close(TMP);
@@ -1325,19 +1540,22 @@ sub get_clientip {
    } else {
       $clientip="127.0.0.1";
    }
-
    return $clientip;
 }
 #################### END GET_CLIENTIP #########################
 
 #################### GET_PROTOCOL #########################
 sub get_protocol {
-   if ($ENV{'HTTPS'}=~/on/i || 
-       $ENV{'SERVER_PORT'}==443 || 
-       $ENV{'HTTP_REFERER'}=~/^https/i ) {
-      return("https");
+   if ($config{'http_protocol'} eq 'auto') {
+      if ($ENV{'HTTPS'}=~/on/i || 
+#         $ENV{'HTTP_REFERER'}=~/^https/i ||
+          $ENV{'SERVER_PORT'}==443 ) {
+         return("https");
+      } else {
+         return("http");
+      }
    } else {
-      return("http");
+      return($config{'http_protocol'});
    }
 }
 #################### END GET_PROTOCOL #########################
@@ -1420,18 +1638,28 @@ sub dateserial2str {
    my $str;
 
    return $serial if ( $serial !~ /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/ );
+
    if ( $format eq "mm/dd/yyyy") {
       $str="$2/$3/$1 $4:$5:$6";
    } elsif ( $format eq "dd/mm/yyyy") {
       $str="$3/$2/$1 $4:$5:$6";
    } elsif ( $format eq "yyyy/mm/dd") {
       $str="$1/$2/$3 $4:$5:$6";
+
    } elsif ( $format eq "mm-dd-yyyy") {
       $str="$2-$3-$1 $4:$5:$6";
    } elsif ( $format eq "dd-mm-yyyy") {
       $str="$3-$2-$1 $4:$5:$6";
    } elsif ( $format eq "yyyy-mm-dd") {
       $str="$1-$2-$3 $4:$5:$6";
+
+   } elsif ( $format eq "mm.dd.yyyy") {
+      $str="$2.$3.$1 $4:$5:$6";
+   } elsif ( $format eq "dd.mm.yyyy") {
+      $str="$3.$2.$1 $4:$5:$6";
+   } elsif ( $format eq "yyyy.mm.dd") {
+      $str="$1.$2.$3 $4:$5:$6";
+
    } else {
       $str="$2/$3/$1 $4:$5:$6";
    }
@@ -1459,6 +1687,8 @@ sub email2nameaddr {
       $address = $1;
       $name =~ s/\@.*$//;
    }
+   $name=~s/^\s+//; $name=~s/\s+$//;
+   $address=~s/^\s+//; $address=~s/\s+$//;
    return($name, $address);
 }
 ################ END EMAIL2NAMEADDR  #####################
@@ -1533,6 +1763,12 @@ sub log_time {
    chmod(0666, "/tmp/openwebmail.debug");
 }
 
+sub is_tainted {
+   return ! eval {
+      join('',@_), kill 0;
+      1;
+   };
+}
 ################## END LOG_TIME (for profiling) ##################
 
 1;
