@@ -1,5 +1,5 @@
 #
-# pop3mail.pl - pop3 mail retrieval routines
+# pop3mail.pl - functions for pop3 mail retrieval
 #
 # 2002/03/19 eddie@turtle.ee.ncku.edu.tw
 #            tung@turtle.ee.ncku.edu.tw
@@ -9,9 +9,6 @@ use strict;
 use Fcntl qw(:DEFAULT :flock);
 use FileHandle;
 use IO::Socket;
-require "mime.pl";
-
-use vars qw(%config);
 
 # return < 0 means error
 # -1 pop3book read error
@@ -37,12 +34,12 @@ sub retrpop3mail {
    }
 
    ($dummy, $dummy, $pop3passwd, $pop3lastid, $pop3del, $enable)=
-			split(/\@\@\@/, $accounts{"$pop3host\@\@\@$pop3user"});
+			split(/\@\@\@/, $accounts{"$pop3host:$pop3user"});
 
    # untaint for file creation
-   ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1);
+   ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1); 	
    # untaint for connection creation
-   ($pop3host =~ /^(.+)$/) && ($pop3host = $1);
+   ($pop3host =~ /^(.+)$/) && ($pop3host = $1); 	
 
    $ServerPort='110';
    eval {
@@ -60,32 +57,21 @@ sub retrpop3mail {
    $_=<$remote_sock>;
    return(-3) if (/^\-/);		# server not ready
 
-   # try if server supports auth login(base64 encoding) first
-   print $remote_sock "auth login\r\n";
+   print $remote_sock "user $pop3user\r\n";
    $_=<$remote_sock>;
-   if (/^\+/) {
-      print $remote_sock &encode_base64($pop3user);
-      $_=<$remote_sock>;
-      (close($remote_sock) && return(-4)) if (/^\-/);		# username error
-      print $remote_sock &encode_base64($pop3passwd);
-      $_=<$remote_sock>;
-   }
-   if (! /^\+/) {	# not supporting auth login or auth login failed
-      print $remote_sock "user $pop3user\r\n";
-      $_=<$remote_sock>;
-      (close($remote_sock) && return(-4)) if (/^\-/);		# username error
-      print $remote_sock "pass $pop3passwd\r\n";
-      $_=<$remote_sock>;
-      (close($remote_sock) && return(-5)) if (/^\-/);		# passwd error
-   }
+   return(-4) if (/^\-/);		# username error
+
+   print $remote_sock "pass $pop3passwd\r\n";
+   $_=<$remote_sock>;
+   return (-5) if (/^\-/);		# passwd error
+
    print $remote_sock "stat\r\n";
    $_=<$remote_sock>;
-   (close($remote_sock) && return(-6)) if (/^\-/);		# stat error
+   return(-6) if (/^\-/);		# stat error
 
    $nMailCount=(split(/\s/))[1];
    if ($nMailCount == 0) {		# no message
       print $remote_sock "quit\r\n";
-      close($remote_sock);
       return 0;
    }
 
@@ -105,7 +91,6 @@ sub retrpop3mail {
          $last=(split(/\s/))[1];		# +OK N
          if ($last eq $nMailCount) {
             print $remote_sock "quit\r\n";
-            close($remote_sock);
             return 0;
          }
       }
@@ -120,7 +105,6 @@ sub retrpop3mail {
 
       if ($pop3lastid eq (split(/\s/))[$uidl_field]) {	# +OK N ID
          print $remote_sock "quit\r\n";
-         close($remote_sock);
          return 0;
       }
       if ($pop3lastid ne "none") {
@@ -153,7 +137,6 @@ sub retrpop3mail {
          if ( /^\+/ ) {
             next;
          } elsif (/^\-/) {
-            close($remote_sock);
             return(-7);
          } else {
             last;
@@ -175,15 +158,13 @@ sub retrpop3mail {
          last if ($_ eq "." );	#end and exit while
          $FileContent .= "$_\n";
          # get $stAddress, $stDate to compose the mail delimiter 'From xxxx' line
-         if ( /\(envelope\-from \s*(.+?)\s*\)/i && $stAddress eq "" ) {
-            $stAddress = $1;
-         } elsif ( /^from:\s+(.+)$/i && $stAddress eq "" ) {
+         if ( /^from:\s+(.+)$/i && $stAddress eq "" ) {
             $_ = $1;
             if ($_=~ /^"?(.+?)"?\s*<(.*)>$/ ) {
                $_ = $2;
             } elsif ($_=~ /<?(.*@.*)>?\s+\((.+?)\)/ ) {
                $_ = $1;
-            } elsif ($_=~ /<\s*(.+@.+)\s*>/ ) {
+            } elsif ($_=~ /<\s*(.+@.+)\s*>/ ) { 
                $_ = $1;
             } else {
                $_=~ s/\s*(.+@.+)\s*/$1/;
@@ -194,26 +175,14 @@ sub retrpop3mail {
             $stDate=$1;
          }
       }
-
-
-      my $dateserial=datefield2dateserial($stDate);
-      my $gmserial=gmtime2dateserial();
-      if ($dateserial eq "" ||
-          dateserial2daydiff($dateserial)-dateserial2daydiff($gmserial)>1 ) {
-         $dateserial=$gmserial;	# use current time if msg time is newer than now
-      }
-      if ($config{'deliver_use_GMT'}) {
-         $stDate=dateserial2delimiter($dateserial, "");
-      } else {
-         $stDate=dateserial2delimiter($dateserial, gettimeoffset());
-      }
+      $stDate=date_for_delimiter($stDate);
 
       # append message to mail folder
       filelock($spoolfile, LOCK_EX);
       open(IN,">>$spoolfile") or return(-8);
       print IN "From $stAddress $stDate\n";
       print IN $FileContent;
-      print IN "\n";		# mark mail end
+      print IN "\n";		# mark mail end 
       close(IN);
       filelock($spoolfile, LOCK_UN);
 
@@ -242,7 +211,39 @@ sub retrpop3mail {
    }
 
    # return number of fetched mail
-   return($retr_total);
+   return($retr_total);		
+}
+
+sub date_for_delimiter {
+   my $datefield=$1;
+
+   my @monthstr=qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
+   my @wdaystr=qw(Sun Mon Tue Wed Thu Fri Sat);
+
+   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =localtime;
+   $year=$year+1900;
+   $mon=$monthstr[$mon]; 
+   $wday=$wdaystr[$wday];
+
+   if ($datefield =~ /(\w+),\s+(\d+)\s+(\w+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s/i) { 
+      #Date: Wed, 9 Sep 1998 19:30:17 +0800 (CST)
+      $wday=$1; $mday=$2; $mon=$3; $year=$4; $hour=$5; $min=$6; $sec=$7;
+   } elsif ($datefield =~ /(\d+)\s+(\w+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s/i) { 
+      #Date: 07 Sep 2000 23:01:36 +0200
+      $mday=$1; $mon=$2; $year=$3; $hour=$4; $min=$5; $sec=$6;
+   } elsif ($datefield =~ /(\w+),\s+(\w+)\s+(\d+),\s+(\d+)\s+(\d+):(\d+):(\d+)\s/i) { 
+      #Date: Wednesday, February 10, 1999 3:39 PM
+      $wday=$1; $mon=$2; $mday=$3; $year=$4; $hour=$5; $min=$6; $sec=$7;
+      $wday=~s/^(...).*/$1/;
+      $mon=~s/^(...).*/$1/;
+   }
+   # some machines has reverse order for month and mday
+   if ( $mday=~/[A-Za-z]+/ ) {
+      my $tmp=$mday; $mday=$mon; $mon=$tmp;
+   }
+
+   return(sprintf("%3s %3s %2d %02d:%02d:%02d %4d",
+              $wday,$mon,$mday, $hour,$min,$sec, $year));
 }
 
 1;
